@@ -1,15 +1,13 @@
 
 from .common import (
     meson_introspect,
-    ninja,
+    meson,
 )
 
 from pathlib import Path
 from contextlib import contextmanager, suppress
 from tempfile import TemporaryDirectory
 
-import os
-
 
 def get_build_opts(build_dir):
     opts = meson_introspect('--buildoptions', cwd=str(build_dir))
@@ -17,7 +15,7 @@ def get_build_opts(build_dir):
 
 
 @contextmanager
-def temp_install(build_dir):
+def temp_install(build_dir, skip_subprojects=True):
     with TemporaryDirectory(prefix='taisei-tempinstall') as tempdir:
         log_path = Path(build_dir) / 'meson-logs' / 'install-log.txt'
         log_bak_path = log_path.with_name(log_path.name + '.bak')
@@ -25,16 +23,16 @@ def temp_install(build_dir):
         prefix = get_build_opts(build_dir)['prefix']['value']
         install_path = Path(tempdir + prefix)
 
-        env = os.environ.copy()
-        env['DESTDIR'] = tempdir
-
         with suppress(FileNotFoundError):
             log_path.rename(log_bak_path)
 
-        try:
-            for target in ('install',): # ('reconfigure', 'install',):
-                ninja(target, cwd=str(build_dir), env=env)
+        meson_cmd = ['install', '--destdir', tempdir]
 
+        if skip_subprojects:
+            meson_cmd += ['--skip-subprojects']
+
+        try:
+            meson(*meson_cmd, cwd=str(build_dir))
             yield install_path
         finally:
             with suppress(FileNotFoundError):
