@@ -2,23 +2,25 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "boss.h"
-#include "global.h"
-#include "stage.h"
-#include "stagetext.h"
-#include "stagedraw.h"
-#include "entity.h"
-#include "util/glm.h"
-#include "portrait.h"
-#include "stages/stage5/stage5.h"  // for unlockable bonus BGM
-#include "stageobjects.h"
+
+#include "audio/audio.h"
 #include "dynstage.h"
+#include "entity.h"
+#include "global.h"
+#include "portrait.h"
+#include "stage.h"
+#include "stagedraw.h"
+#include "stageobjects.h"
+#include "stages/stage5/stage5.h"  // for unlockable bonus BGM
+#include "stagetext.h"
+#include "util/env.h"
+#include "util/glm.h"
+#include "util/graphics.h"
 
 #define DAMAGE_PER_POWER_POINT 500.0f
 #define DAMAGE_PER_POWER_ITEM  (DAMAGE_PER_POWER_POINT * POWER_VALUE)
@@ -52,7 +54,7 @@ TASK(boss_damage_to_power, { BoxedBoss boss; }) {
 }
 
 Boss *create_boss(const char *name, char *ani, cmplx pos) {
-	Boss *boss = objpool_acquire(&stage_object_pools.bosses);
+	auto boss = STAGE_ACQUIRE_OBJ(Boss);
 
 	boss->name = name;
 	boss->pos = pos;
@@ -439,15 +441,13 @@ static void draw_spell_name(Boss *b, int time, bool healthbar_radial) {
 	float opacity = opacity_noplr * b->hud.plrproximity_opacity;
 
 	r_draw_sprite(&(SpriteParams) {
-		.sprite = "spell",
+		.sprite_ptr = res_sprite("spell"),
+		.shader_ptr = res_shader("sprite_default"),
 		.pos = { (VIEWPORT_W - 128), y_offset * (1 - pow(1 - f2, 5)) + VIEWPORT_H * pow(1 - f2, 2) },
 		.color = color_mul_scalar(RGBA(1, 1, 1, f2 * 0.5), opacity * f2) ,
 		.scale.both = 3 - 2 * (1 - pow(1 - f2, 3)),
 	});
 
-	bool cullcap_saved = r_capability_current(RCAP_CULL_FACE);
-	r_disable(RCAP_CULL_FACE);
-
 	int delay = attacktype_start_delay(b->current->type);
 	float warn_progress = clamp((time + delay) / 120.0, 0, 1);
 
@@ -471,8 +471,6 @@ static void draw_spell_name(Boss *b, int time, bool healthbar_radial) {
 
 	r_mat_mv_pop();
 
-	r_capability(RCAP_CULL_FACE, cullcap_saved);
-
 	if(warn_progress < 1) {
 		draw_spell_warning(font, im(x0) - font_get_lineskip(font), warn_progress, opacity);
 	}
@@ -655,6 +653,7 @@ DEFINE_TASK(boss_particles) {
 				.timeout = 180,
 				.draw_rule = pdraw_timeout_scale(2, 0.01),
 				.angle = rng_angle(),
+				.flags = PFLAG_MANUALANGLE,
 			));
 		}
 
@@ -684,7 +683,9 @@ void draw_boss_background(Boss *boss) {
 
 	r_mat_mv_scale(f, f, 1);
 	r_draw_sprite(&(SpriteParams) {
-		.sprite = "boss_circle",
+		.sprite_ptr = res_sprite("boss_circle"),
+		.shader_ptr = res_shader("sprite_particle"),
+		.shader_params = &(ShaderCustomParams) { 1.0f },
 		.color = RGBA(1, 1, 1, 0),
 	});
 	r_mat_mv_pop();
@@ -714,6 +715,8 @@ static void ent_draw_boss(EntityInterface *ent) {
 
 	r_draw_sprite(&(SpriteParams) {
 		.sprite_ptr = aniplayer_get_frame(&boss->ani),
+		.shader_ptr = res_shader("sprite_particle"),
+		.shader_params = &(ShaderCustomParams) { 1.0f },
 		.pos.as_cmplx = boss->pos + boss_get_sprite_offset(boss),
 		.color = c,
 	});
@@ -728,15 +731,9 @@ void draw_boss_fake_overlay(Boss *boss) {
 void draw_boss_overlay(Boss *boss) {
 	bool radial_style = healthbar_style_is_radial();
 
-	if(!radial_style) {
-		draw_linear_healthbar(boss);
-	}
-
 	float o = boss->hud.global_opacity * boss->hud.plrproximity_opacity;
 
 	if(o > 0) {
-		draw_boss_text(ALIGN_LEFT, 10, 20 + 8 * !radial_style, boss->name, res_font("standard"), RGBA(o, o, o, o));
-
 		if(boss->current && ATTACK_IS_SPELL(boss->current->type)) {
 			int t_portrait, t_spell;
 			t_portrait = t_spell = global.frames - boss->current->starttime;
@@ -746,6 +743,8 @@ void draw_boss_overlay(Boss *boss) {
 			draw_spell_name(boss, t_spell, radial_style);
 		}
 
+		draw_boss_text(ALIGN_LEFT, 10, 20 + 8 * !radial_style, boss->name, res_font("standard"), RGBA(o, o, o, o));
+
 		float remaining = boss->hud.attack_timer;
 		Color clr_int, clr_fract;
 
@@ -810,6 +809,10 @@ void draw_boss_overlay(Boss *boss) {
 
 		r_color4(1, 1, 1, 1);
 	}
+
+	if(!radial_style) {
+		draw_linear_healthbar(boss);
+	}
 }
 
 static void boss_rule_extra(Boss *boss, float alpha) {
@@ -1354,7 +1357,7 @@ void free_boss(Boss *boss) {
 	ent_unregister(&boss->ent);
 	boss_set_portrait(boss, NULL, NULL, NULL);
 	aniplayer_free(&boss->ani);
-	objpool_release(&stage_object_pools.bosses, boss);
+	STAGE_RELEASE_OBJ(boss);
 }
 
 static void boss_schedule_next_attack(Boss *b, Attack *a) {
