@@ -2,8 +2,8 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
 #pragma once
@@ -36,6 +36,8 @@ struct CLIAction {
 	CutsceneID cutscene;
 	bool force_intro;
 	bool unlock_all;
+	int width;
+	int height;
 };
 
 int cli_args(int argc, char **argv, CLIAction *a);
