@@ -21,8 +21,8 @@
 #include "replay/state.h"
 #include "stageinfo.h"
 
-#include <SDL.h>
-#include <SDL_platform.h>
+#include <SDL3/SDL.h>
+#include <SDL3/SDL_platform.h>
 
 enum {
 	// defaults
