@@ -2,25 +2,20 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "laser.h"
 #include "internal.h"
 #include "draw.h"
 
 #include "global.h"
 #include "list.h"
-#include "stageobjects.h"
-#include "stagedraw.h"
 #include "renderer/api.h"
-#include "resource/model.h"
-#include "util/fbmgr.h"
+#include "stage.h"
+#include "stageobjects.h"
 #include "util/glm.h"
-#include "video.h"
 
 typedef struct LaserSamplingParams {
 	uint num_samples;
@@ -43,9 +38,7 @@ Laser *create_laser(
 	LaserPosRule prule,
 	cmplx a0, cmplx a1, cmplx a2, cmplx a3
 ) {
-	Laser *l = objpool_acquire(&stage_object_pools.lasers);
-	alist_push(&global.lasers, l);
-
+	auto l = alist_push(&global.lasers, STAGE_ACQUIRE_OBJ(Laser));
 	l->birthtime = global.frames;
 	l->timespan = time;
 	l->deathtime = deathtime;
@@ -98,7 +91,7 @@ void laserline_set_posdir(Laser *l, cmplx pos, cmplx dir) {
 static void *_delete_laser(ListAnchor *lasers, List *laser, void *arg) {
 	Laser *l = (Laser*)laser;
 	ent_unregister(&l->ent);
-	objpool_release(&stage_object_pools.lasers, alist_unlink(lasers, laser));
+	STAGE_RELEASE_OBJ(alist_unlink(lasers, l));
 	return NULL;
 }
 
