@@ -2,8 +2,8 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
 #pragma once
@@ -11,6 +11,10 @@
 
 #include "menu.h"
 
+typedef struct IngameMenuContext {
+	const char *title;
+} IngameMenuContext;
+
 void draw_ingame_menu_bg(MenuData *menu, float f);
 void draw_ingame_menu(MenuData *menu);
 
