@@ -19,14 +19,15 @@ enum {
 
 static const uint8_t pxi_magic[] = { 0xe2, 0x99, 0xa5, 0x52, 0x65, 0x69, 0x6d, 0x75 };
 
-static bool px_internal_probe(SDL_RWops *stream) {
+static bool px_internal_probe(SDL_IOStream *stream) {
 	uint8_t magic[sizeof(pxi_magic)] = { 0 };
-	SDL_RWread(stream, magic, sizeof(magic), 1);
+	SDL_ReadIO(stream, magic, sizeof(magic));
 	return !memcmp(magic, pxi_magic, sizeof(magic));
 }
 
-static bool px_internal_save(SDL_RWops *stream, const Pixmap *pixmap, const PixmapSaveOptions *opts) {
-	SDL_RWops *cstream = NULL;
+static bool px_internal_save(SDL_IOStream *stream, const Pixmap *pixmap,
+			     const PixmapSaveOptions *opts) {
+	SDL_IOStream *cstream = NULL;
 
 	#define CHECK_WRITE(expr, expected_size) \
 		do { \
@@ -36,9 +37,9 @@ static bool px_internal_save(SDL_RWops *stream, const Pixmap *pixmap, const Pixm
 			} \
 		} while(0)
 
-	#define W_BYTES(stream, data, size) CHECK_WRITE(SDL_RWwrite(stream, data, size, 1), size)
-	#define W_U32(stream, val) CHECK_WRITE(SDL_WriteLE32(stream, val), sizeof(uint32_t))
-	#define W_U16(stream, val) CHECK_WRITE(SDL_WriteLE16(stream, val), sizeof(uint16_t))
+	#define W_BYTES(stream, data, size) CHECK_WRITE(SDL_WriteIO(stream, data, size) == size, size)
+	#define W_U32(stream, val) CHECK_WRITE(SDL_WriteU32LE(stream, val), sizeof(uint32_t))
+	#define W_U16(stream, val) CHECK_WRITE(SDL_WriteU16LE(stream, val), sizeof(uint16_t))
 	#define W_U8(stream, val)  CHECK_WRITE(SDL_WriteU8(stream, val), sizeof(uint8_t))
 
 	W_BYTES(stream, pxi_magic, sizeof(pxi_magic));
@@ -54,7 +55,7 @@ static bool px_internal_save(SDL_RWops *stream, const Pixmap *pixmap, const Pixm
 	W_U8(cstream, pixmap->origin);
 	W_BYTES(cstream, pixmap->data.untyped, pixmap->data_size);
 
-	SDL_RWclose(cstream);
+	SDL_CloseIO(cstream);
 	cstream = NULL;
 
 	W_U32(stream, crc32);
@@ -63,26 +64,31 @@ static bool px_internal_save(SDL_RWops *stream, const Pixmap *pixmap, const Pixm
 
 fail:
 	if(cstream) {
-		SDL_RWclose(cstream);
+		SDL_CloseIO(cstream);
 	}
 
 	return false;
 }
 
-static bool px_internal_load(SDL_RWops *stream, Pixmap *pixmap, PixmapFormat preferred_format) {
+static bool px_internal_load(SDL_IOStream *stream, Pixmap *pixmap,
+			     PixmapFormat preferred_format) {
 	assume(pixmap->data.untyped == NULL);
 
-	SDL_RWops *cstream = NULL;
+	SDL_IOStream *cstream = NULL;
+
+	// TODO check for read errors
 
 	uint8_t magic[sizeof(pxi_magic)] = { 0 };
-	SDL_RWread(stream, magic, 1, sizeof(magic));
+	SDL_ReadIO(stream, magic, sizeof(magic));
 
 	if(memcmp(magic, pxi_magic, sizeof(magic))) {
 		log_error("Bad magic header");
 		return false;
 	}
 
-	uint16_t version = SDL_ReadLE16(stream);
+	uint16_t version = 0;
+	SDL_ReadU16LE(stream, &version);
+
 	if(version != PXI_VERSION) {
 		log_error("Bad version %u; expected %u", version, PXI_VERSION);
 		return false;
@@ -93,11 +99,18 @@ static bool px_internal_load(SDL_RWops *stream, Pixmap *pixmap, PixmapFormat pre
 
 	// TODO validate and verify consistency of data_size/width/height/format
 
-	pixmap->width = SDL_ReadLE32(cstream);
-	pixmap->height = SDL_ReadLE32(cstream);
-	pixmap->data_size = SDL_ReadLE32(cstream);
-	pixmap->format = SDL_ReadLE16(cstream);
-	pixmap->origin = SDL_ReadU8(cstream);
+	SDL_ReadU32LE(cstream, &pixmap->width);
+	SDL_ReadU32LE(cstream, &pixmap->height);
+	SDL_ReadU32LE(cstream, &pixmap->data_size);
+
+	uint16_t tmp_u16 = 0;
+	uint8_t tmp_u8 = 0;
+
+	SDL_ReadU16LE(cstream, &tmp_u16);
+	pixmap->format = tmp_u16;
+
+	SDL_ReadU8(cstream, &tmp_u8);
+	pixmap->origin = tmp_u8;
 
 	if(pixmap->data_size > PIXMAP_BUFFER_MAX_SIZE) {
 		log_error("Data size is too large");
@@ -113,17 +126,23 @@ static bool px_internal_load(SDL_RWops *stream, Pixmap *pixmap, PixmapFormat pre
 	}
 
 	pixmap->data.untyped = mem_alloc(pixmap->data_size);
-	size_t read = SDL_RWread(cstream, pixmap->data.untyped, 1, pixmap->data_size);
+	size_t read = SDL_ReadIO(cstream, pixmap->data.untyped, pixmap->data_size);
+
+	if(read == 0 && SDL_GetIOStatus(cstream) != SDL_IO_STATUS_EOF) {
+		log_sdl_error(LOG_ERROR, "SDL_ReadIO");
+		goto fail;
+	}
 
 	if(read != pixmap->data_size) {
 		log_error("Read %zu bytes, expected %u", read, pixmap->data_size);
 		goto fail;
 	}
 
-	SDL_RWclose(cstream);
+	SDL_CloseIO(cstream);
 	cstream = NULL;
 
-	uint32_t crc32_in = SDL_ReadLE32(stream);
+	uint32_t crc32_in = 0;
+	SDL_ReadU32LE(stream, &crc32_in);
 
 	if(crc32_in != crc32) {
 		log_error("CRC32 mismatch: 0x%08x != 0x%08x", crc32_in, crc32);
@@ -139,7 +158,7 @@ fail:
 	}
 
 	if(cstream) {
-		SDL_RWclose(cstream);
+		SDL_CloseIO(cstream);
 	}
 
 	return false;
