@@ -2,18 +2,20 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
+#include "marisa.h"
 
+#include "audio/audio.h"
+#include "common_tasks.h"
+#include "dialog/marisa.h"
 #include "global.h"
 #include "plrmodes.h"
-#include "marisa.h"
 #include "renderer/api.h"
 #include "stagedraw.h"
-#include "common_tasks.h"
+#include "util/graphics.h"
 
 #define SHOT_FORWARD_DAMAGE 60
 #define SHOT_FORWARD_DELAY 6
