@@ -2,17 +2,19 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
+#include "reimu.h"
 
+#include "audio/audio.h"
+#include "common_tasks.h"
+#include "dialog/reimu.h"
 #include "global.h"
 #include "plrmodes.h"
-#include "reimu.h"
 #include "stagedraw.h"
-#include "common_tasks.h"
+#include "util/graphics.h"
 
 #define SHOT_FORWARD_DMG 35
 #define SHOT_FORWARD_DELAY 6
@@ -280,9 +282,9 @@ static void reimu_dream_draw_gaps(EntityInterface *gap_renderer_ent) {
 	r_uniform_vec2("viewport", VIEWPORT_W, VIEWPORT_H);
 	r_uniform_float("time", global.frames / 60.0f);
 	r_uniform_vec2("gap_size", GAP_WIDTH/2.0f, GAP_LENGTH/2.0f);
-	r_uniform_vec2_array("gaps[0]", 0, NUM_GAPS, gaps);
-	r_uniform_float_array("gap_angles[0]", 0, NUM_GAPS, angles);
-	r_uniform_int_array("gap_links[0]", 0, NUM_GAPS, links);
+	r_uniform_vec2_array("gaps", 0, NUM_GAPS, gaps);
+	r_uniform_float_array("gap_angles", 0, NUM_GAPS, angles);
+	r_uniform_int_array("gap_links", 0, NUM_GAPS, links);
 	draw_framebuffer_tex(framebuffers->front, VIEWPORT_W, VIEWPORT_H);
 
 	r_blend(BLEND_PREMUL_ALPHA);
