@@ -2,17 +2,19 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
+#include "youmu.h"
 
+#include "audio/audio.h"
+#include "dialog/youmu.h"
 #include "global.h"
 #include "plrmodes.h"
-#include "youmu.h"
 #include "renderer/api.h"
 #include "stagedraw.h"
+#include "util/graphics.h"
 
 #define SHOT_SELF_DELAY 6
 #define SHOT_SELF_DAMAGE 60
