@@ -2,12 +2,10 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "projectile.h"
 
 #include "global.h"
@@ -247,7 +245,7 @@ static Projectile* _create_projectile(ProjArgs *args) {
 		log_fatal("Tried to spawn a projectile while in drawing code");
 	}
 
-	Projectile *p = objpool_acquire(&stage_object_pools.projectiles);
+	auto p = STAGE_ACQUIRE_OBJ(Projectile);
 
 	p->birthtime = global.frames;
 	p->pos = p->pos0 = p->prevpos = args->pos;
@@ -328,7 +326,7 @@ static void delete_projectile(ProjectileList *projlist, Projectile *p, ProjColli
 	signal_event_with_collision_result(p, &p->events.killed, col);
 	COEVENT_CANCEL_ARRAY(p->events);
 	ent_unregister(&p->ent);
-	objpool_release(&stage_object_pools.projectiles, alist_unlink(projlist, p));
+	STAGE_RELEASE_OBJ(alist_unlink(projlist, p));
 }
 
 static void *foreach_delete_projectile(ListAnchor *projlist, List *proj, void *arg) {
