@@ -39,6 +39,7 @@ typedef enum RendererFeature {
 	RFEAT_TEXTURE_BOTTOMLEFT_ORIGIN,
 	RFEAT_TEXTURE_SWIZZLE,
 	RFEAT_PARTIAL_MIPMAPS,
+	RFEAT_DEFAULT_FRAMEBUFFER_READBACK,
 
 	NUM_RFEATS,
 } RendererFeature;
@@ -548,7 +549,7 @@ CullFaceMode r_cull_current(void);
 void r_depth_func(DepthTestFunc func);
 DepthTestFunc r_depth_func_current(void);
 
-bool r_shader_language_supported(const ShaderLangInfo *lang, ShaderLangInfo *out_alternative) attr_nonnull(1);
+bool r_shader_language_supported(const ShaderLangInfo *lang, SPIRVTranspileOptions *transpile_opts) attr_nonnull(1);
 
 ShaderObject* r_shader_object_compile(ShaderSource *source) attr_nonnull(1);
 void r_shader_object_destroy(ShaderObject *shobj) attr_nonnull(1);
@@ -790,7 +791,7 @@ const char* r_vertex_buffer_get_debug_label(VertexBuffer *vbuf) attr_nonnull(1);
 void r_vertex_buffer_set_debug_label(VertexBuffer *vbuf, const char* label) attr_nonnull(1);
 void r_vertex_buffer_destroy(VertexBuffer *vbuf) attr_nonnull(1);
 void r_vertex_buffer_invalidate(VertexBuffer *vbuf) attr_nonnull(1);
-SDL_RWops* r_vertex_buffer_get_stream(VertexBuffer *vbuf) attr_nonnull(1);
+SDL_IOStream* r_vertex_buffer_get_stream(VertexBuffer *vbuf) attr_nonnull(1);
 
 IndexBuffer* r_index_buffer_create(uint index_size, size_t max_elements);
 size_t r_index_buffer_get_capacity(IndexBuffer *ibuf) attr_nonnull(1);
@@ -819,12 +820,13 @@ void r_scissor(int x, int y, int w, int h);
 void r_scissor_rect(IntRect scissor);
 void r_scissor_current(IntRect *scissor) attr_nonnull(1);
 
-void r_model_add_static(Model *out_mdl, Primitive prim, size_t num_vertices, GenericModelVertex vertices[num_vertices], size_t num_indices, uint32_t indices[num_indices]);
+void r_model_add_static(Model *out_mdl, Primitive prim, size_t num_vertices, GenericModelVertex vertices[], size_t num_indices, uint32_t indices[]);
 const Model *r_model_get_quad(void) attr_returns_nonnull;
 
 void r_vsync(VsyncMode mode);
 VsyncMode r_vsync_current(void);
 
+void r_begin_frame(void);
 void r_swap(SDL_Window *window);
 
 void r_mat_mv_push(void);
