@@ -8,6 +8,7 @@
 
 #include "shaderlib.h"
 #include "cache.h"
+#include "reflect.h"
 
 #include "log.h"
 #include "util/sha256.h"
@@ -16,32 +17,44 @@
 #include "rwops/rwops_zstd.h"
 #include "vfs/public.h"
 
-#define CACHE_VERSION 5
+#define CACHE_VERSION 8
 #define CRC_INIT 0
 
 #define MAX_CONTENT_SIZE          (1024 * 1024)
-#define MAX_GLSL_ATTRIBS          255
 #define MAX_GLSL_MACROS           255
 #define MAX_GLSL_MACRO_NAME_LEN   255
 #define MAX_GLSL_MACRO_VALUE_LEN  255
+#define MAX_ENTRYPOINT_NAME_LEN   255
 
 static uint8_t *shader_cache_construct_entry(const ShaderSource *src, const ShaderMacro *macros, size_t *out_size) {
 	uint8_t *buf, *result = NULL;
 	uint32_t crc = CRC_INIT;
 
-	SDL_RWops *abuf = SDL_RWAutoBuffer((void**)&buf, BUFSIZ);
-	SDL_RWops *crcbuf = SDL_RWWrapCRC32(abuf, &crc, false);
-	SDL_RWops *dest = crcbuf;
+	SDL_IOStream *abuf = SDL_RWAutoBuffer((void**)&buf, BUFSIZ);
+	SDL_IOStream *crcbuf = SDL_RWWrapCRC32(abuf, &crc, false);
+	SDL_IOStream *dest = crcbuf;
 
 	if(UNLIKELY(!crcbuf)) {
 		log_sdl_error(LOG_ERROR, "SDL_RWWrapCRC32");
 		return NULL;
 	}
 
+	// TODO: check for IO errors
+
 	SDL_WriteU8(dest, CACHE_VERSION);
 	SDL_WriteU8(dest, src->stage);
 	SDL_WriteU8(dest, src->lang.lang);
 
+	size_t entrypoint_name_len = strlen(src->entrypoint);
+
+	if(entrypoint_name_len > MAX_ENTRYPOINT_NAME_LEN) {
+		log_error("Entry point name is too long (%zu): %s", entrypoint_name_len, src->entrypoint);
+		goto fail;
+	}
+
+	SDL_WriteU8(dest, entrypoint_name_len);
+	SDL_WriteIO(dest, src->entrypoint, entrypoint_name_len);
+
 	if(macros) {
 		uint num_macros = 0;
 
@@ -71,10 +84,10 @@ static uint8_t *shader_cache_construct_entry(const ShaderSource *src, const Shad
 			}
 
 			SDL_WriteU8(dest, name_len);
-			SDL_RWwrite(dest, m->name, name_len, 1);
+			SDL_WriteIO(dest, m->name, name_len);
 
 			SDL_WriteU8(dest, value_len);
-			SDL_RWwrite(dest, m->value, value_len, 1);
+			SDL_WriteIO(dest, m->value, value_len);
 		}
 	} else {
 		SDL_WriteU8(dest, 0);
@@ -82,29 +95,8 @@ static uint8_t *shader_cache_construct_entry(const ShaderSource *src, const Shad
 
 	switch(src->lang.lang) {
 		case SHLANG_GLSL: {
-			SDL_WriteLE16(dest, src->lang.glsl.version.version);
+			SDL_WriteU16LE(dest, src->lang.glsl.version.version);
 			SDL_WriteU8(dest, src->lang.glsl.version.profile);
-
-			if(src->meta.glsl.num_attributes > MAX_GLSL_ATTRIBS) {
-				log_error("Too many attributes (%i)", src->meta.glsl.num_attributes);
-				goto fail;
-			}
-
-			SDL_WriteU8(dest, src->meta.glsl.num_attributes);
-
-			for(uint i = 0; i < src->meta.glsl.num_attributes; ++i) {
-				SDL_WriteLE32(dest, src->meta.glsl.attributes[i].location);
-
-				size_t len = strlen(src->meta.glsl.attributes[i].name);
-
-				if(len > 255) {
-					log_error("Attribute name is too long (%zu): %s", len, src->meta.glsl.attributes[i].name);
-					goto fail;
-				}
-
-				SDL_WriteU8(dest, len);
-				SDL_RWwrite(dest, src->meta.glsl.attributes[i].name, len, 1);
-			}
 			break;
 		}
 
@@ -113,8 +105,22 @@ static uint8_t *shader_cache_construct_entry(const ShaderSource *src, const Shad
 			break;
 		}
 
+		case SHLANG_HLSL: {
+			SDL_WriteU8(dest, src->lang.hlsl.shader_model);
+			break;
+		}
+
+		case SHLANG_DXBC: {
+			SDL_WriteU8(dest, src->lang.dxbc.shader_model);
+			break;
+		}
+
+		case SHLANG_MSL: {
+			break;
+		}
+
 		default: {
-			log_error("Unhandled shading language id=%u", src->lang.lang);
+			log_error("Unhandled shading language %s", shader_lang_name(src->lang.lang));
 			goto fail;
 		}
 	}
@@ -124,121 +130,158 @@ static uint8_t *shader_cache_construct_entry(const ShaderSource *src, const Shad
 		goto fail;
 	}
 
-	SDL_WriteLE32(dest, src->content_size);
-	SDL_RWwrite(dest, src->content, src->content_size, 1);
+	if(!shader_reflection_serialize(src->reflection, dest)) {
+		goto fail;
+	}
+
+	SDL_WriteU32LE(dest, src->content_size);
+	SDL_WriteIO(dest, src->content, src->content_size);
 
 	dest = abuf;
-	SDL_RWclose(crcbuf);
+	SDL_CloseIO(crcbuf);
 	crcbuf = NULL;
 
-	SDL_WriteLE32(dest, crc);
+	SDL_WriteU32LE(dest, crc);
 
-	*out_size = SDL_RWtell(dest);
+	*out_size = SDL_TellIO(dest);
 	result = memdup(buf, *out_size);
 
 fail:
 	if(crcbuf != NULL) {
-		SDL_RWclose(crcbuf);
+		SDL_CloseIO(crcbuf);
 	}
 
-	SDL_RWclose(abuf);
+	SDL_CloseIO(abuf);
 	return result;
 }
 
-static bool shader_cache_load_entry(SDL_RWops *stream, ShaderSource *out_src) {
+#define READ(_file, _func, _type) ({ \
+	_type _tmp = 0; \
+	if(UNLIKELY(!(_func)((_file), &_tmp))) { \
+		log_error("Read error: %s", SDL_GetError()); \
+	} \
+	_tmp; \
+})
+
+#define READU8(_file)    READ(_file, SDL_ReadU8,    uint8_t)
+#define READU16LE(_file) READ(_file, SDL_ReadU16LE, uint16_t)
+#define READU32LE(_file) READ(_file, SDL_ReadU32LE, uint32_t)
+
+static bool shader_cache_load_entry(SDL_IOStream *stream, ShaderSource *out_src, MemArena *arena) {
 	uint32_t crc = CRC_INIT;
-	SDL_RWops *s = SDL_RWWrapCRC32(stream, &crc, false);
+	SDL_IOStream *s = SDL_RWWrapCRC32(stream, &crc, false);
+
+	auto arena_snap = marena_snapshot(arena);
 
 	memset(out_src, 0, sizeof(*out_src));
 	out_src->content = NULL;
 
-	if(SDL_ReadU8(s) != CACHE_VERSION) {
+	// TODO check for IO errors
+
+	if(READU8(s) != CACHE_VERSION) {
 		log_error("Version mismatch");
 		goto fail;
 	}
 
-	out_src->stage = SDL_ReadU8(s);
+	out_src->stage = READU8(s);
 
 	memset(&out_src->lang, 0, sizeof(out_src->lang));
-	out_src->lang.lang = SDL_ReadU8(s);
+	out_src->lang.lang = READU8(s);
 
-	if(SDL_ReadU8(s) > 0) {
+	uint32_t entrypoint_len = READU8(s);
+	char *entrypoint = marena_alloc(arena, entrypoint_len + 1);
+	entrypoint[entrypoint_len] = 0;
+
+	if(SDL_ReadIO(s, entrypoint, entrypoint_len) != entrypoint_len) {
+		log_sdl_error(LOG_ERROR, "SDL_ReadIO");
+		goto fail;
+	}
+
+	out_src->entrypoint = entrypoint;
+
+	if(READU8(s) > 0) {
 		log_error("Cache entry contains macros, this is not implemented");
 		goto fail;
 	}
 
 	switch(out_src->lang.lang) {
 		case SHLANG_GLSL: {
-			out_src->lang.glsl.version.version = SDL_ReadLE16(s);
-			out_src->lang.glsl.version.profile = SDL_ReadU8(s);
-
-			uint nattrs = SDL_ReadU8(s);
-
-			if(nattrs > 0) {
-				out_src->meta.glsl.num_attributes = nattrs;
-				out_src->meta.glsl.attributes = ALLOC_ARRAY(nattrs, typeof(*out_src->meta.glsl.attributes));
-
-				for(uint i = 0; i < nattrs; ++i) {
-					GLSLAttribute *attr = out_src->meta.glsl.attributes + i;
-					attr->location = SDL_ReadLE32(s);
-					uint len = SDL_ReadU8(s);
-					attr->name = mem_alloc(len + 1);
-					SDL_RWread(s, attr->name, len, 1);
-				}
-			}
+			out_src->lang.glsl.version.version = READU16LE(s);
+			out_src->lang.glsl.version.profile = READU8(s);
 			break;
 		}
 
 		case SHLANG_SPIRV: {
-			out_src->lang.spirv.target = SDL_ReadU8(s);
+			out_src->lang.spirv.target = READU8(s);
+			break;
+		}
+
+		case SHLANG_HLSL: {
+			out_src->lang.hlsl.shader_model = READU8(s);
+			break;
+		}
+
+		case SHLANG_DXBC: {
+			out_src->lang.dxbc.shader_model = READU8(s);
+			break;
+		}
+
+		case SHLANG_MSL: {
 			break;
 		}
 
 		default: {
-			log_error("Unhandled shading language id=%u", out_src->lang.lang);
+			log_error("Unhandled shading language %s", shader_lang_name(out_src->lang.lang));
 			goto fail;
 		}
 	}
 
-	out_src->content_size = SDL_ReadLE32(s);
-	out_src->content = mem_alloc(out_src->content_size);
-
-	if(SDL_RWread(s, out_src->content, out_src->content_size, 1) != 1) {
-		log_error("Read error");
+	if(!shader_reflection_deserialize(&out_src->reflection, arena, s)) {
 		goto fail;
 	}
 
-	uint32_t file_crc = SDL_ReadLE32(stream);
+	uint32_t content_size = READU32LE(s);
+	char *content = marena_alloc(arena, content_size);
+
+	if(SDL_ReadIO(s, content, content_size) != content_size) {
+		log_sdl_error(LOG_ERROR, "SDL_ReadIO");
+		goto fail;
+	}
+
+	out_src->content_size = content_size;
+	out_src->content = content;
+
+	uint32_t file_crc = READU32LE(stream);
 
 	if(crc != file_crc) {
 		log_error("CRC mismatch (%08x != %08x), cache entry is corrupted", crc, file_crc);
 		goto fail;
 	}
 
-	SDL_RWclose(s);
+	SDL_CloseIO(s);
 	return true;
 
 fail:
-	shader_free_source(out_src);
-	SDL_RWclose(s);
+	marena_rollback(arena, &arena_snap);
+	SDL_CloseIO(s);
 	return false;
 }
 
-bool shader_cache_get(const char *hash, const char *key, ShaderSource *entry) {
+bool shader_cache_get(const char *hash, const char *key, ShaderSource *entry, MemArena *arena) {
 	char path[256];
 	snprintf(path, sizeof(path), "cache/shaders/%s/%s", hash, key);
 
-	SDL_RWops *stream = vfs_open(path, VFS_MODE_READ);
+	SDL_IOStream *stream = vfs_open(path, VFS_MODE_READ);
 
 	if(stream == NULL) {
 		return false;
 	}
 
 	stream = NOT_NULL(SDL_RWWrapZstdReader(stream, true));
-	bool result = shader_cache_load_entry(stream, entry);
-	SDL_RWclose(stream);
+	bool result = shader_cache_load_entry(stream, entry, arena);
+	SDL_CloseIO(stream);
 
-	log_debug("Retrieved %s/%s from cache", hash, key);
+	log_debug("%s %s/%s from cache", result ? "Retrieved " : "Failed to retrieve",  hash, key);
 	return result;
 }
 
@@ -250,7 +293,7 @@ static bool shader_cache_set_raw(const char *hash, const char *key, uint8_t *ent
 	vfs_mkdir(path);
 	snprintf(path, sizeof(path), "cache/shaders/%s/%s", hash, key);
 
-	SDL_RWops *out = vfs_open(path, VFS_MODE_WRITE);
+	SDL_IOStream *out = vfs_open(path, VFS_MODE_WRITE);
 
 	if(out == NULL) {
 		log_error("VFS error: %s", vfs_get_error());
@@ -258,8 +301,8 @@ static bool shader_cache_set_raw(const char *hash, const char *key, uint8_t *ent
 	}
 
 	out = NOT_NULL(SDL_RWWrapZstdWriter(out, RW_ZSTD_LEVEL_DEFAULT, true));
-	SDL_RWwrite(out, entry, entry_size, 1);
-	SDL_RWclose(out);
+	SDL_WriteIO(out, entry, entry_size);
+	SDL_CloseIO(out);
 
 	return true;
 }
