@@ -11,6 +11,8 @@
 
 #include "defs.h"
 
+#include "memory/arena.h"
+
 // sha256 hexdigest  : 64 bytes
 // separator         : 1 byte
 // 64-bit size (hex) : 8 bytes
@@ -20,7 +22,7 @@
 bool shader_cache_hash(const ShaderSource *src, const ShaderMacro *macros, size_t buf_size, char out_buf[buf_size])
 	attr_nonnull(1, 4) attr_nodiscard;
 
-bool shader_cache_get(const char *hash, const char *key, ShaderSource *out_src)
+bool shader_cache_get(const char *hash, const char *key, ShaderSource *entry, MemArena *arena)
 	attr_nonnull(1, 2, 3) attr_nodiscard;
 
 bool shader_cache_set(const char *hash, const char *key, const ShaderSource *src)
