@@ -10,7 +10,8 @@
 #include "shaderlib.h"
 
 #include "log.h"
-#include "rwops/rwops_autobuf.h"
+#include "memory/scratch.h"
+#include "rwops/rwops_arena.h"
 #include "util/io.h"
 #include "util/stringops.h"
 #include "vfs/pathutil.h"
@@ -26,10 +27,10 @@ enum {
 typedef struct GLSLParseState {
 	const GLSLSourceOptions *options;
 	ShaderSource *src;
-	SDL_RWops *dest;
+	SDL_IOStream *dest;
+	MemArena *scratch;
+	RWArenaState stream_state;
 	bool version_defined;
-	char *linebuf;
-	size_t linebuf_size;
 } GLSLParseState;
 
 typedef struct GLSLFileParseState {
@@ -154,7 +155,7 @@ static bool glsl_process_file(GLSLFileParseState *fstate) {
 		return false;
 	}
 
-	SDL_RWops *stream;
+	SDL_IOStream *stream;
 	GLSLSourceOpenCallback open = fstate->global->options->file_open_callback;
 
 	if(open) {
@@ -163,7 +164,7 @@ static bool glsl_process_file(GLSLFileParseState *fstate) {
 		stream = vfs_open(fstate->path, VFS_MODE_READ);
 	}
 
-	SDL_RWops *dest = fstate->global->dest;
+	SDL_IOStream *dest = fstate->global->dest;
 
 	if(!stream) {
 		log_error("VFS error: %s", vfs_get_error());
@@ -175,20 +176,25 @@ static bool glsl_process_file(GLSLFileParseState *fstate) {
 	++fstate->lineno;
 	glsl_write_lineno(fstate);
 
-	while(SDL_RWgets_realloc(stream, &fstate->global->linebuf, &fstate->global->linebuf_size)) {
-		char *p = fstate->global->linebuf;
+	auto scratch = fstate->global->scratch;
+	auto scratch_snap = marena_snapshot(scratch);
+
+	char *line;
+
+	while(*(line = SDL_RWgets_arena(stream, scratch, NULL))) {
+		char *p = line;
 		skip_space(&p);
 
 		if(check_directive(p, INCLUDE_DIRECTIVE)) {
 			if(!glsl_try_write_header(fstate)) {
-				SDL_RWclose(stream);
+				SDL_CloseIO(stream);
 				return false;
 			}
 
 			char *filename = glsl_parse_include_directive(fstate, p);
 
 			if(filename == NULL) {
-				SDL_RWclose(stream);
+				SDL_CloseIO(stream);
 				return false;
 			}
 
@@ -207,7 +213,7 @@ static bool glsl_process_file(GLSLFileParseState *fstate) {
 			SDL_RWprintf(dest, "// end include (level %i)\n", fstate->include_level);
 
 			if(!include_ok) {
-				SDL_RWclose(stream);
+				SDL_CloseIO(stream);
 				return false;
 			}
 
@@ -215,14 +221,14 @@ static bool glsl_process_file(GLSLFileParseState *fstate) {
 		} else if(check_directive(p, VERSION_DIRECTIVE)) {
 			if(fstate->global->version_defined) {
 				log_error("%s:%d: duplicate or misplaced %s directive", fstate->path, fstate->lineno, VERSION_DIRECTIVE);
-				SDL_RWclose(stream);
+				SDL_CloseIO(stream);
 				return false;
 			}
 
 			GLSLVersion shader_v = glsl_parse_version_directive(fstate, p);
 
 			if(shader_v.version == 0) {
-				SDL_RWclose(stream);
+				SDL_CloseIO(stream);
 				return false;
 			}
 
@@ -257,53 +263,52 @@ static bool glsl_process_file(GLSLFileParseState *fstate) {
 			glsl_write_header(fstate);
 		} else {
 			if(*p && !glsl_try_write_header(fstate)) {
-				SDL_RWclose(stream);
+				SDL_CloseIO(stream);
 				return false;
 			}
 
-			SDL_RWwrite(dest, fstate->global->linebuf, 1, strlen(fstate->global->linebuf));
+			SDL_WriteIO(dest, line, strlen(line));
 		}
 
 		fstate->lineno++;
+
 		glsl_write_lineno(fstate);
+		marena_rollback(scratch, &scratch_snap);
 	}
 
-	SDL_RWclose(stream);
+	SDL_CloseIO(stream);
 	return true;
 }
 
-bool glsl_load_source(const char *path, ShaderSource *out, const GLSLSourceOptions *options) {
-	void *bufdata_ptr;
-	SDL_RWops *out_buf = SDL_RWAutoBuffer(&bufdata_ptr, 1024);
-	assert(out_buf != NULL);
-
+bool glsl_load_source(const char *path, ShaderSource *out, MemArena *arena, const GLSLSourceOptions *options) {
 	memset(out, 0, sizeof(*out));
 	out->lang.lang = SHLANG_GLSL;
 	out->lang.glsl.version = options->version;
 	out->stage = options->stage;
+	out->entrypoint = "main";
 
 	GLSLParseState pstate = { 0 };
-	pstate.dest = out_buf;
+	pstate.dest = NOT_NULL(SDL_RWArena(arena, 1024, &pstate.stream_state));
 	pstate.src = out;
 	pstate.options = options;
-	pstate.linebuf_size = 128;
-	pstate.linebuf = mem_alloc(pstate.linebuf_size);
+	pstate.scratch = acquire_scratch_arena();
 
 	GLSLFileParseState fstate = { 0 };
 	fstate.global = &pstate;
 	fstate.path = path;
 
 	bool result = glsl_process_file(&fstate);
-	mem_free(pstate.linebuf);
+	release_scratch_arena(pstate.scratch);
 
 	if(result) {
-		SDL_WriteU8(out_buf, 0);
-		out->content_size = strlen(bufdata_ptr) + 1;
-		out->content = mem_alloc(out->content_size);
-		memcpy(out->content, bufdata_ptr, out->content_size);
+		SDL_WriteU8(pstate.dest, 0);
+		out->content = pstate.stream_state.buffer;
+		out->content_size = pstate.stream_state.io_offset;
+		// log_debug("*****\n%s\n*****", out->content);
+		assert(strlen(out->content) == out->content_size - 1);
 	}
 
-	SDL_RWclose(out_buf);
+	SDL_CloseIO(pstate.dest);
 	return result;
 }
 
@@ -371,17 +376,6 @@ char *glsl_parse_version(const char *str, GLSLVersion *out_version) {
 	return p;
 }
 
-void glsl_free_source(ShaderSource *src) {
-	ShaderSourceMetaGLSL *m = &src->meta.glsl;
-
-	for(uint i = 0; i < m->num_attributes; ++i) {
-		mem_free(m->attributes[i].name);
-	}
-
-	mem_free(m->attributes);
-	m->attributes = NULL;
-}
-
 bool glsl_version_supports_instanced_rendering(GLSLVersion v) {
 	if(v.profile == GLSL_PROFILE_ES) {
 		return v.version >= 300;
