@@ -10,8 +10,9 @@
 #include "taisei.h"
 
 #include "defs.h"
+#include "memory/arena.h"
 
-#include <SDL_rwops.h>
+#include <SDL3/SDL_iostream.h>
 
 typedef enum GLSLProfile {
 	GLSL_PROFILE_NONE,
@@ -29,17 +30,10 @@ typedef struct ShaderLangInfoGLSL {
 	GLSLVersion version;
 } ShaderLangInfoGLSL;
 
-typedef struct GLSLAttribute {
-	char *name;
-	uint location;
-} GLSLAttribute;
-
 typedef struct ShaderSourceMetaGLSL {
-	uint num_attributes;
-	GLSLAttribute *attributes;
 } ShaderSourceMetaGLSL;
 
-typedef SDL_RWops *(*GLSLSourceOpenCallback)(const char *path, void *userdata);
+typedef SDL_IOStream *(*GLSLSourceOpenCallback)(const char *path, void *userdata);
 
 typedef struct GLSLSourceOptions {
 	GLSLVersion version;
@@ -50,8 +44,7 @@ typedef struct GLSLSourceOptions {
 	void *file_open_callback_userdata;
 } GLSLSourceOptions;
 
-bool glsl_load_source(const char *path, ShaderSource *out, const GLSLSourceOptions *options) attr_nonnull(1, 2, 3);
+bool glsl_load_source(const char *path, ShaderSource *out, MemArena *arena, const GLSLSourceOptions *options) attr_nonnull(1, 2, 3);
 char *glsl_parse_version(const char *str, GLSLVersion *out_version) attr_nonnull(1, 2);
 int glsl_format_version(char *buf, size_t bufsize, GLSLVersion version) attr_nonnull(1);
-void glsl_free_source(ShaderSource *src) attr_nonnull(1);
 bool glsl_version_supports_instanced_rendering(GLSLVersion v);
