@@ -8,20 +8,6 @@
 
 #include "shaderlib.h"
 
-void shader_free_source(ShaderSource *src) {
-	switch(src->lang.lang) {
-		case SHLANG_GLSL:
-			glsl_free_source(src);
-			break;
-
-		default: break;
-	}
-
-	mem_free(src->content);
-	src->content = NULL;
-	src->content_size = 0;
-}
-
 bool shader_lang_supports_uniform_locations(const ShaderLangInfo *lang) {
 	if(lang->lang == SHLANG_SPIRV) {
 		return true;
@@ -38,18 +24,16 @@ bool shader_lang_supports_uniform_locations(const ShaderLangInfo *lang) {
 	}
 }
 
-bool shader_lang_supports_attribute_locations(const ShaderLangInfo *lang) {
-	if(lang->lang == SHLANG_SPIRV) {
-		return true;
+const char *shader_lang_name(ShaderLanguage lang) {
+	switch(lang) {
+		case SHLANG_INVALID: return "INVALID";
+		case SHLANG_GLSL:    return "GLSL";
+		case SHLANG_SPIRV:   return "SPIR-V";
+		case SHLANG_HLSL:    return "HLSL";
+		case SHLANG_DXBC:    return "DXBC";
+		case SHLANG_MSL:     return "MSL";
 	}
 
-	if(lang->lang != SHLANG_GLSL) {
-		return false; // FIXME?
-	}
-
-	if(lang->glsl.version.profile == GLSL_PROFILE_ES) {
-		return lang->glsl.version.version >= 300;
-	} else {
-		return lang->glsl.version.version >= 330;
-	}
+	assert(0 && "Unknown ShaderLanguage value");
+	return "UNKNOWN";
 }
