@@ -11,47 +11,29 @@
 
 #include "opengl.h"
 #include "../api.h"
+#include "../common/cached_buffer.h"
 
 typedef struct CommonBuffer CommonBuffer;
 
 struct CommonBuffer {
-	union {
-		SDL_RWops stream;
-		struct {
-			char padding[offsetof(SDL_RWops, hidden)];
+	CachedBuffer cachedbuf;
 
-			struct {
-				char *buffer;
-				size_t update_begin;
-				size_t update_end;
-			} cache;
+	size_t commited_size;
+	GLuint gl_handle;
+	GLuint gl_usage_hint;
+	uint bindidx;
+	char debug_label[R_DEBUG_LABEL_SIZE];
 
-			size_t offset;
-			size_t size;
-			size_t commited_size;
-			GLuint gl_handle;
-			GLuint gl_usage_hint;
-			uint bindidx;
-			char debug_label[R_DEBUG_LABEL_SIZE];
-
-			void (*pre_bind)(CommonBuffer *self);
-			void (*post_bind)(CommonBuffer *self);
-		};
-	};
+	void (*pre_bind)(CommonBuffer *self);
+	void (*post_bind)(CommonBuffer *self);
 };
 
-static_assert(
-	offsetof(CommonBuffer, stream) == 0,
-	"stream should be the first member in CommonBuffer for simplicity"
-);
-
 CommonBuffer *gl33_buffer_create(uint bindidx, size_t alloc_size);
-void gl33_buffer_init_cache(CommonBuffer *cbuf, size_t capacity);
 void gl33_buffer_init(CommonBuffer *cbuf, size_t capacity, void *data, GLenum usage_hint);
 void gl33_buffer_destroy(CommonBuffer *cbuf);
 void gl33_buffer_invalidate(CommonBuffer *cbuf);
 void gl33_buffer_resize(CommonBuffer *cbuf, size_t new_size);
-SDL_RWops *gl33_buffer_get_stream(CommonBuffer *cbuf);
+SDL_IOStream *gl33_buffer_get_stream(CommonBuffer *cbuf);
 void gl33_buffer_flush(CommonBuffer *cbuf);
 
 #define GL33_BUFFER_TEMP_BIND(cbuf, code) do { \
