@@ -11,7 +11,7 @@
 #include "../glcommon/debug.h"
 #include "../glcommon/shaders.h"
 
-bool gl33_shader_language_supported(const ShaderLangInfo *lang, ShaderLangInfo *out_alternative) {
+bool gl33_shader_language_supported(const ShaderLangInfo *lang, SPIRVTranspileOptions *transpile_opts) {
 	if(glcommon_shader_lang_table.data) {
 		dynarray_foreach_elem(&glcommon_shader_lang_table, ShaderLangInfo *l, {
 			if(!memcmp(l, lang, sizeof(*l))) {
@@ -19,8 +19,9 @@ bool gl33_shader_language_supported(const ShaderLangInfo *lang, ShaderLangInfo *
 			}
 		});
 
-		if(out_alternative) {
-			*out_alternative = dynarray_get(&glcommon_shader_lang_table, 0);
+		if(transpile_opts) {
+			transpile_opts->compile.target = SPIRV_TARGET_OPENGL_450;
+			transpile_opts->decompile.lang = &dynarray_get(&glcommon_shader_lang_table, 0);
 		}
 
 		return false;
@@ -30,10 +31,16 @@ bool gl33_shader_language_supported(const ShaderLangInfo *lang, ShaderLangInfo *
 
 	bool supported = lang->lang == SHLANG_GLSL;
 
-	if(!supported && out_alternative) {
-		out_alternative->lang = SHLANG_GLSL;
-		out_alternative->glsl.version.version = 330;
-		out_alternative->glsl.version.profile = GLSL_PROFILE_CORE;
+	if(!supported && transpile_opts) {
+		static const ShaderLangInfo fallback_info = {
+			.lang = SHLANG_GLSL,
+			.glsl.version = {
+				.version = 330,
+				GLSL_PROFILE_CORE,
+			}
+		};
+		transpile_opts->compile.target = SPIRV_TARGET_OPENGL_450;
+		transpile_opts->decompile.lang = &fallback_info;
 	}
 
 	return supported;
@@ -95,24 +102,12 @@ ShaderObject *gl33_shader_object_compile(ShaderSource *source) {
 	ShaderObject *shobj = NULL;
 
 	if(status) {
-		uint nattribs = source->meta.glsl.num_attributes;
-
 		shobj = ALLOC(ShaderObject, {
 			.gl_handle = gl_handle,
 			.stage = source->stage,
-			.num_attribs = nattribs,
 		});
+
 		snprintf(shobj->debug_label, sizeof(shobj->debug_label), "Shader object #%i", gl_handle);
-
-		if(nattribs > 0) {
-			shobj->attribs = ALLOC_ARRAY(nattribs, typeof(*shobj->attribs));
-		}
-
-		for(uint i = 0; i < nattribs; ++i) {
-			GLSLAttribute *a = source->meta.glsl.attributes + i;
-			shobj->attribs[i].name = mem_strdup(a->name);
-			shobj->attribs[i].location = a->location;
-		}
 	} else {
 		glDeleteShader(gl_handle);
 	}
@@ -122,14 +117,6 @@ ShaderObject *gl33_shader_object_compile(ShaderSource *source) {
 
 void gl33_shader_object_destroy(ShaderObject *shobj) {
 	glDeleteShader(shobj->gl_handle);
-
-	uint nattribs = shobj->num_attribs;
-
-	for(uint i = 0; i < nattribs; ++i) {
-		mem_free(shobj->attribs[i].name);
-	}
-
-	mem_free(shobj->attribs);
 	mem_free(shobj);
 }
 
@@ -149,13 +136,6 @@ bool gl33_shader_object_transfer(ShaderObject *dst, ShaderObject *src) {
 
 	glDeleteShader(dst->gl_handle);
 
-	uint nattribs = dst->num_attribs;
-
-	for(uint i = 0; i < nattribs; ++i) {
-		mem_free(dst->attribs[i].name);
-	}
-
-	mem_free(dst->attribs);
 	*dst = *src;
 	mem_free(src);
 
