@@ -18,11 +18,9 @@ struct ShaderObject {
 	GLuint gl_handle;
 	ShaderStage stage;
 	char debug_label[R_DEBUG_LABEL_SIZE];
-	uint num_attribs;
-	GLSLAttribute *attribs;
 };
 
-bool gl33_shader_language_supported(const ShaderLangInfo *lang, ShaderLangInfo *out_alternative);
+bool gl33_shader_language_supported(const ShaderLangInfo *lang, SPIRVTranspileOptions *transpile_opts);
 
 ShaderObject *gl33_shader_object_compile(ShaderSource *source);
 void gl33_shader_object_destroy(ShaderObject *shobj);
