@@ -2,17 +2,17 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "texture.h"
+
 #include "../api.h"
-#include "opengl.h"
-#include "gl33.h"
 #include "../glcommon/debug.h"
+#include "gl33.h"
+#include "opengl.h"
+#include "util.h"
 
 static GLenum class_to_gltarget(TextureClass cls) {
 	switch(cls) {
@@ -229,28 +229,29 @@ static void gl33_texture_set(Texture *tex, uint mipmap, uint layer, const Pixmap
 
 	GLenum gl_target = target_from_class_and_layer(tex->params.class, layer);
 	GLenum ifmt = tex->fmt_info->internal_format;
+	GLenum xfmt = xfer->gl_format;
+	GLenum xtype = xfer->gl_type;
 
 	if(tex->fmt_info->flags & GLTEX_COMPRESSED) {
-		glCompressedTexImage2D(
+		glCompressedTexSubImage2D(
 			gl_target,
 			mipmap,
-			ifmt,
+			0,
+			0,
 			width,
 			height,
-			0,
+			ifmt,
 			image->data_size,
 			image_data
 		);
 	} else {
-		GLenum xfmt = xfer->gl_format;
-		GLenum xtype = xfer->gl_type;
-		glTexImage2D(
+		glTexSubImage2D(
 			gl_target,
 			mipmap,
-			tex->fmt_info->internal_format,
+			0,
+			0,
 			width,
 			height,
-			0,
 			xfmt,
 			xtype,
 			image_data
@@ -394,7 +395,9 @@ Texture *gl33_texture_create(const TextureParams *params) {
 	GLenum xfmt = xfer->gl_format;
 	GLenum xtype = xfer->gl_type;
 
-	for(uint i = 0; i < p->mipmaps; ++i) {
+	if(glext.texture_storage) {
+		glTexStorage2D(gl_target, p->mipmaps, ifmt, p->width, p->height);
+	} else for(uint i = 0; i < p->mipmaps; ++i) {
 		uint w, h;
 		gl33_texture_get_size(tex, i, &w, &h);
 
@@ -457,6 +460,25 @@ void gl33_texture_invalidate(Texture *tex) {
 		return;
 	}
 
+	if(glext.texture_storage) {
+		if(glext.invalidate_subdata) {
+#ifndef STATIC_GLES3
+			gl33_bind_texture(tex, 0, -1);
+			gl33_sync_texunit(tex->binding_unit, false, true);
+
+			for(uint i = 0; i < tex->params.mipmaps; ++i) {
+				glInvalidateTexImage(tex->bind_target, i);
+			}
+#else
+		UNREACHABLE;
+#endif
+		} else {
+			log_debug("TODO/FIXME: can't invalidate immutable texture without GL_ARB_invalidate_subdata!");
+		}
+
+		return;
+	}
+
 	gl33_bind_texture(tex, 0, -1);
 	gl33_sync_texunit(tex->binding_unit, false, true);
 
