@@ -11,7 +11,7 @@
 
 #include "util/callchain.h"
 
-#include <SDL.h>
+#include <SDL3/SDL.h>
 
 #define REPLAY_EXTENSION "tsr"
 
@@ -35,8 +35,8 @@ typedef enum ReplayReadMode {
 void replay_reset(Replay *rpy) attr_nonnull_all;
 void replay_destroy_events(Replay *rpy) attr_nonnull_all;
 
-bool replay_write(Replay *rpy, SDL_RWops *file, uint16_t version) attr_nonnull_all;
-bool replay_read(Replay *rpy, SDL_RWops *file, ReplayReadMode mode, const char *source) attr_nonnull(1, 2);
+bool replay_write(Replay *rpy, SDL_IOStream *file, uint16_t version) attr_nonnull_all;
+bool replay_read(Replay *rpy, SDL_IOStream *file, ReplayReadMode mode, const char *source) attr_nonnull(1, 2);
 
 bool replay_save(Replay *rpy, const char *name) attr_nonnull_all;
 bool replay_save_syspath(Replay *rpy, const char *path, uint16_t version) attr_nonnull_all;
