@@ -13,7 +13,9 @@
 
 uint32_t replay_struct_stage_metadata_checksum(ReplayStage *stg, uint16_t version);
 
-SDL_RWops *replay_wrap_stream_compress(uint16_t version, SDL_RWops *rw, bool autoclose);
-SDL_RWops *replay_wrap_stream_decompress(uint16_t version, SDL_RWops *rw, bool autoclose);
+SDL_IOStream *replay_wrap_stream_compress(uint16_t version, SDL_IOStream *rw,
+					  bool autoclose);
+SDL_IOStream *replay_wrap_stream_decompress(uint16_t version,
+					    SDL_IOStream *rw, bool autoclose);
 
 extern uint8_t replay_magic_header[REPLAY_MAGIC_HEADER_SIZE];
