@@ -2,15 +2,13 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "sfxbgm_common.h"
 #include "audio/backend.h"
-#include "util.h"
+#include "util/io.h"
 
 static const char *const *get_exts(bool isbgm, uint *numexts) {
 	const char *const *exts = (isbgm
