@@ -9,6 +9,6 @@
 #pragma once
 #include "taisei.h"
 
-#include <SDL.h>
+#include <SDL3/SDL.h>
 
-SDL_RWops *SDL_RWAutoBuffer(void **ptr, size_t initsize);
+SDL_IOStream *SDL_RWAutoBuffer(void **ptr, size_t initsize);
