@@ -16,9 +16,9 @@ int64_t rwutil_compute_seek_pos(int64_t offset, int whence, int64_t pos, int64_t
 	assert(size < 0 || pos <= size);
 
 	switch(whence) {
-		case RW_SEEK_SET: new_pos = offset;        break;
-		case RW_SEEK_CUR: new_pos = pos + offset;  break;
-		case RW_SEEK_END:
+		case SDL_IO_SEEK_SET : new_pos = offset;        break;
+		case SDL_IO_SEEK_CUR : new_pos = pos + offset;  break;
+		case SDL_IO_SEEK_END :
 			if(size < 0) {
 				return -1;
 			}
@@ -40,23 +40,26 @@ int64_t rwutil_compute_seek_pos(int64_t offset, int whence, int64_t pos, int64_t
 }
 
 int64_t rwutil_seek_emulated(
-	SDL_RWops *rw, int64_t offset, int whence,
-	int64_t *pos, rwutil_reopen_func reopen, size_t readbuf_size, void *readbuf
+	SDL_IOStream *rw, int64_t offset, int whence,
+	int64_t *pos, rwutil_reopen_func reopen, void *reopen_arg,
+	size_t readbuf_size, void *readbuf
 ) {
-	int64_t sz = SDL_RWsize(rw);
+	int64_t sz = SDL_GetIOSize(rw);
 	int64_t new_pos = rwutil_compute_seek_pos(offset, whence, *pos, sz);
 
 	if(new_pos < 0) {
 		assert(sz < 0);
-		return sz;  // assume SDL_RWsize set an error;
+		return sz;  // assume SDL_GetIOSize set an error;
 	}
 
-	return rwutil_seek_emulated_abs(rw, new_pos, pos, reopen, readbuf_size, readbuf);
+	return rwutil_seek_emulated_abs(rw, new_pos, pos, reopen, reopen_arg, readbuf_size, readbuf);
 }
 
 int64_t rwutil_seek_emulated_abs(
-	SDL_RWops *rw, int64_t new_pos, int64_t *pos,
-	rwutil_reopen_func reopen, size_t readbuf_size, void *readbuf
+	SDL_IOStream *rw,
+	int64_t new_pos, int64_t *pos,
+	rwutil_reopen_func reopen, void *reopen_arg,
+	size_t readbuf_size, void *readbuf
 ) {
 	assert(new_pos >= 0);
 
@@ -70,9 +73,13 @@ int64_t rwutil_seek_emulated_abs(
 
 	while(new_pos > *pos) {
 		size_t want_read_size = min(new_pos - *pos, readbuf_size);
-		size_t read_size = SDL_RWread(rw, readbuf, 1, want_read_size);
+		size_t read_size = SDL_ReadIO(rw, readbuf, want_read_size);
 		assert(read_size <= want_read_size);
 
+		if(read_size == 0 && SDL_GetIOStatus(rw) != SDL_IO_STATUS_EOF) {
+			return -1;
+		}
+
 		if(read_size < readbuf_size) {
 			break;
 		}
