@@ -312,6 +312,7 @@ static void gameover_menu_result(CallChainResult ccr) {
 
 	switch(action) {
 		case GAMEOVERMENU_ACTION_QUIT:
+		case GAMEOVERMENU_ACTION_DEFAULT:
 			if(global.plr.stats.total.continues_used >= MAX_CONTINUES) {
 				global.gameover = GAMEOVER_DEFEAT;
 			} else {
@@ -480,10 +481,10 @@ static void stage_do_quickload(StageFrameState *fstate) {
 static bool stage_input_handler_gameplay(SDL_Event *event, void *arg) {
 	StageFrameState *fstate = NOT_NULL(arg);
 
-	if(event->type == SDL_KEYDOWN && !event->key.repeat && is_quicksave_allowed()) {
-		if(event->key.keysym.scancode == config_get_int(CONFIG_KEY_QUICKSAVE)) {
+	if(event->type == SDL_EVENT_KEY_DOWN && !event->key.repeat && is_quicksave_allowed()) {
+		if(event->key.scancode == config_get_int(CONFIG_KEY_QUICKSAVE)) {
 			stage_do_quicksave(fstate, false);
-		} else if(event->key.keysym.scancode == config_get_int(CONFIG_KEY_QUICKLOAD)) {
+		} else if(event->key.scancode == config_get_int(CONFIG_KEY_QUICKLOAD)) {
 			stage_do_quickload(fstate);
 		}
 
@@ -532,7 +533,7 @@ static bool stage_input_handler_replay(SDL_Event *event, void *arg) {
 }
 
 static bool stage_input_handler_demo(SDL_Event *event, void *arg) {
-	if(event->type == SDL_KEYDOWN && !event->key.repeat) {
+	if(event->type == SDL_EVENT_KEY_DOWN && !event->key.repeat) {
 		goto exit;
 	}
 
