@@ -2,20 +2,23 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
-#include "global.h"
 #include "stagedraw.h"
-#include "stagetext.h"
-#include "video.h"
-#include "resource/postprocess.h"
+
 #include "entity.h"
-#include "util/fbmgr.h"
+#include "events.h"
+#include "global.h"
 #include "replay/struct.h"
+#include "resource/postprocess.h"
+#include "stageobjects.h"
+#include "stagetext.h"
+#include "util/env.h"
+#include "util/fbmgr.h"
+#include "util/graphics.h"
+#include "video.h"
 
 #ifdef DEBUG
 	#define GRAPHS_DEFAULT 1
@@ -82,6 +85,10 @@ static struct {
 	}
 };
 
+bool stage_draw_is_initialized(void) {
+	return stagedraw.mfb_group;
+}
+
 static double fb_scale(void) {
 	float vp_width, vp_height;
 	video_get_viewport_size(&vp_width, &vp_height);
@@ -530,8 +537,8 @@ static void draw_spellbg(int t) {
 	}
 
 	r_draw_sprite(&(SpriteParams) {
-		.sprite = "boss_spellcircle0",
-		.shader = "sprite_default",
+		.sprite_ptr = res_sprite("boss_spellcircle0"),
+		.shader_ptr = res_shader("sprite_default"),
 		.pos = { re(b->pos), im(b->pos) },
 		.rotation.angle = global.frames * 7.0 * DEG2RAD,
 		.rotation.vector = { 0, 0, -1 },
@@ -567,6 +574,10 @@ static bool fxaa_rule(Framebuffer *fb) {
 	r_depth_func(DEPTH_ALWAYS);
 	r_blend(BLEND_NONE);
 	r_shader_ptr(stagedraw.shaders.fxaa);
+	Texture *tex = r_framebuffer_get_attachment(fb, FRAMEBUFFER_ATTACH_COLOR0);
+	uint w, h;
+	r_texture_get_size(tex, 0, &w, &h);
+	r_uniform_vec2("tex_size", w, h);
 	r_uniform_sampler("depth", r_framebuffer_get_attachment(fb, FRAMEBUFFER_ATTACH_DEPTH));
 	draw_framebuffer_tex(fb, VIEWPORT_W, VIEWPORT_H);
 	r_state_pop();
@@ -611,6 +622,7 @@ static bool draw_powersurge_effect(Framebuffer *target_fb, BlendMode blend) {
 	r_state_push();
 	r_blend(BLEND_NONE);
 	r_disable(RCAP_DEPTH_TEST);
+	r_disable(RCAP_CULL_FACE);
 
 	// if(player_is_powersurge_active(&global.plr)) {
 		r_state_push();
@@ -664,6 +676,7 @@ static bool boss_distortion_rule(Framebuffer *fb) {
 	r_state_push();
 	r_blend(BLEND_NONE);
 	r_disable(RCAP_DEPTH_TEST);
+	r_disable(RCAP_CULL_FACE);
 
 	cmplx fpos = global.boss->pos;;
 	cmplx pos = fpos;
@@ -798,6 +811,7 @@ static void stage_render_bg(StageInfo *stage) {
 	if(should_draw_stage_bg()) {
 		r_mat_mv_push();
 		r_enable(RCAP_DEPTH_TEST);
+		r_enable(RCAP_CULL_FACE);
 		stage->procs->draw();
 		r_mat_mv_pop();
 		fbpair_swap(background);
@@ -805,6 +819,7 @@ static void stage_render_bg(StageInfo *stage) {
 
 	set_ortho(VIEWPORT_W, VIEWPORT_H);
 	r_disable(RCAP_DEPTH_TEST);
+	r_disable(RCAP_CULL_FACE);
 
 	apply_bg_shaders(stage->procs->shader_rules, background);
 
@@ -865,20 +880,20 @@ static void stage_draw_objects(void) {
 
 void stage_draw_overlay(void) {
 	r_state_push();
-	r_shader("sprite_default");
 	r_blend(BLEND_PREMUL_ALPHA);
+	r_disable(RCAP_CULL_FACE);
 
 	if(global.boss) {
 		draw_boss_overlay(global.boss);
 	}
 
+	player_draw_overlay(&global.plr);
+
 	if(stagedraw.clear_screen.alpha > 0) {
 		fade_out(stagedraw.clear_screen.alpha * 0.5);
 	}
 
-	r_shader_standard();
 	stagetext_draw();
-	player_draw_overlay(&global.plr);
 	r_state_pop();
 }
 
@@ -1160,8 +1175,8 @@ static void stage_draw_hud_scores(float ypos_hiscore, float ypos_score, char *bu
 
 static void stage_draw_hud_objpool_stats(float x, float y, float width) {
 	char buf[128];
-	auto objpools = STAGE_OBJPOOLS_AS_ARRAYPTR;
-	MemArena *arena = (*objpools)[0].arena;
+	auto objpools = &stage_objects.pools.as_array;
+	MemArena *arena = &stage_objects.arena;
 
 	Font *font = res_font("monotiny");
 
@@ -1190,12 +1205,20 @@ static void stage_draw_hud_objpool_stats(float x, float y, float width) {
 
 	y += lineskip * 1.5;
 
+	const char *const names[] = {
+		#define GET_POOL_NAME(t, n) #t,
+		OBJECT_POOLS(GET_POOL_NAME)
+		#undef GET_POOL_NAME
+	};
+
+	static_assert(ARRAY_SIZE(*objpools) == ARRAY_SIZE(names));
+
 	for(int i = 0; i < ARRAY_SIZE(*objpools); ++i) {
-		ObjectPool *p = &(*objpools)[i];
+		MemPool *p = &(*objpools)[i];
 
 		snprintf(buf, sizeof(buf), "%u | %7u", p->num_used, p->num_allocated);
 
-		text_draw(p->tag, &(TextParams) {
+		text_draw(names[i], &(TextParams) {
 			.pos = { x, y },
 			.font_ptr = font,
 			.align = ALIGN_LEFT,
@@ -1561,7 +1584,7 @@ void stage_draw_bottom_text(void) {
 		r_uniform_vec3("color_low",  1.0, 0.0, 0.0);
 		r_uniform_vec3("color_mid",  1.0, 1.0, 0.0);
 		r_uniform_vec3("color_high", 0.0, 1.0, 0.0);
-		r_uniform_float_array("points[0]", 0, graphspan, graph);
+		r_uniform_float_array("points", 0, graphspan, graph);
 		draw_graph(142, SCREEN_H - text_h, graphspan, text_h);
 	}
 #endif
@@ -1590,7 +1613,7 @@ static void stage_draw_framerate_graphs(float x, float y, float w, float h) {
 	r_uniform_vec3("color_low",  0.0, 1.0, 1.0);
 	r_uniform_vec3("color_mid",  1.0, 1.0, 0.0);
 	r_uniform_vec3("color_high", 1.0, 0.0, 0.0);
-	r_uniform_float_array("points[0]", 0, NUM_SAMPLES, samples);
+	r_uniform_float_array("points", 0, NUM_SAMPLES, samples);
 	draw_graph(x, y, w, h);
 
 	// x -= w * 1.1;
@@ -1600,7 +1623,7 @@ static void stage_draw_framerate_graphs(float x, float y, float w, float h) {
 	r_uniform_vec3("color_low",  0.0, 1.0, 0.0);
 	r_uniform_vec3("color_mid",  1.0, 0.0, 0.0);
 	r_uniform_vec3("color_high", 1.0, 0.0, 0.5);
-	r_uniform_float_array("points[0]", 0, NUM_SAMPLES, samples);
+	r_uniform_float_array("points", 0, NUM_SAMPLES, samples);
 	draw_graph(x, y, w, h);
 
 	r_state_pop();
@@ -1721,10 +1744,10 @@ void stage_draw_hud(void) {
 
 	// Difficulty indicator
 	r_draw_sprite(&(SpriteParams) {
-		.sprite = difficulty_sprite_name(global.diff),
+		.sprite_ptr = res_sprite(difficulty_sprite_name(global.diff)),
 		.pos = { HUD_EFFECTIVE_WIDTH * 0.5, 400 },
 		.scale.both = 0.6,
-		.shader = "sprite_default",
+		.shader_ptr = res_shader("sprite_default"),
 	});
 
 	// Power/Item/Voltage icons
@@ -1733,41 +1756,41 @@ void stage_draw_hud(void) {
 
 	r_draw_sprite(&(SpriteParams) {
 		.pos = { 2, labels.y.power + 2 },
-		.sprite = "item/power",
-		.shader = "sprite_default",
+		.sprite_ptr = res_sprite("item/power"),
+		.shader_ptr = res_shader("sprite_default"),
 		.color = RGBA(0, 0, 0, 0.5),
 	});
 
 	r_draw_sprite(&(SpriteParams) {
 		.pos = { 0, labels.y.power },
-		.sprite = "item/power",
-		.shader = "sprite_default",
+		.sprite_ptr = res_sprite("item/power"),
+		.shader_ptr = res_shader("sprite_default"),
 	});
 
 	r_draw_sprite(&(SpriteParams) {
 		.pos = { 2, labels.y.value + 2 },
-		.sprite = "item/point",
-		.shader = "sprite_default",
+		.sprite_ptr = res_sprite("item/point"),
+		.shader_ptr = res_shader("sprite_default"),
 		.color = RGBA(0, 0, 0, 0.5),
 	});
 
 	r_draw_sprite(&(SpriteParams) {
 		.pos = { 0, labels.y.value },
-		.sprite = "item/point",
-		.shader = "sprite_default",
+		.sprite_ptr = res_sprite("item/point"),
+		.shader_ptr = res_shader("sprite_default"),
 	});
 
 	r_draw_sprite(&(SpriteParams) {
 		.pos = { 2, labels.y.voltage + 2 },
-		.sprite = "item/voltage",
-		.shader = "sprite_default",
+		.sprite_ptr = res_sprite("item/voltage"),
+		.shader_ptr = res_shader("sprite_default"),
 		.color = RGBA(0, 0, 0, 0.5),
 	});
 
 	r_draw_sprite(&(SpriteParams) {
 		.pos = { 0, labels.y.voltage },
-		.sprite = "item/voltage",
-		.shader = "sprite_default",
+		.sprite_ptr = res_sprite("item/voltage"),
+		.shader_ptr = res_shader("sprite_default"),
 	});
 
 	r_mat_mv_pop();
@@ -1809,8 +1832,8 @@ void stage_draw_hud(void) {
 			red = 0;
 
 		r_draw_sprite(&(SpriteParams) {
-			.sprite = "boss_indicator",
-			.shader = "sprite_default",
+			.sprite_ptr = res_sprite("boss_indicator"),
+			.shader_ptr = res_shader("sprite_default"),
 			.pos = { VIEWPORT_X+re(global.boss->pos), 590 },
 			.color = RGBA(1 - red, 1 - red, 1 - red, 1 - red),
 		});
@@ -1855,7 +1878,7 @@ void stage_draw_hud(void) {
 		text_draw("Demo", &(TextParams) {
 			.align = ALIGN_CENTER,
 			.font = "big",
-			.shader = "text_demo",
+			.shader_ptr = res_shader("text_demo"),
 			.shader_params = &(ShaderCustomParams) { global.frames / 60.0f },
 			.pos.as_cmplx = pos,
 		});
