@@ -2,15 +2,15 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
-*/
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
+ */
 
 #pragma once
 #include "taisei.h"
 
+#include "stageinfo.h"
 #include "util/fbpair.h"
-#include "stagedraw.h"
 
 typedef struct Stage1DrawData {
 	FBPair water_fbpair;
