@@ -2,19 +2,14 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
-*/
-
-#include "taisei.h"
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
+ */
 
 #include "spells.h"
-#include "../cirno.h"
 
 #ifdef SPELL_BENCHMARK
 
-#include "global.h"
-
 static void stage1_spell_benchmark_proc(Boss *b, int t) {
 	int N = 5000; // number of particles on the screen
 
