@@ -2,25 +2,20 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
+#include "timeline.h"   // IWYU pragma: keep
 
-#include "timeline.h"
-#include "stage3.h"
-#include "wriggle.h"
+#include "draw.h"
+#include "nonspells/nonspells.h"
 #include "scuttle.h"
 #include "spells/spells.h"
-#include "nonspells/nonspells.h"
-#include "background_anim.h"
-#include "draw.h"
+#include "stage3.h"
+#include "wriggle.h"
 
-#include "global.h"
-#include "stagetext.h"
-#include "common_tasks.h"
-#include "enemy_classes.h"
+#include "stages/common_imports.h"
 
 static void stage3_dialog_post_boss(void) {
 	PlayerMode *pm = global.plr.mode;
