@@ -2,25 +2,18 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
-#include "stage4.h"
 #include "timeline.h"
-#include "draw.h"
-#include "background_anim.h"
-#include "nonspells/nonspells.h"
-#include "kurumi.h"
 
-#include "global.h"
-#include "stage.h"
-#include "enemy.h"
-#include "enemy_classes.h"
-#include "laser.h"
-#include "common_tasks.h"
+#include "background_anim.h"
+#include "kurumi.h"
+#include "nonspells/nonspells.h"
+#include "stage4.h"
+
+#include "stages/common_imports.h"
 #include "../stage6/elly.h"
 
 static void stage4_dialog_pre_boss(void) {
