@@ -2,20 +2,15 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "timeline.h"
-#include "stage5.h"
+
 #include "background_anim.h"
 #include "nonspells/nonspells.h"
-#include "spells/spells.h"
-
-#include "coroutine.h"
-#include "enemy_classes.h"
+#include "stage5.h"
 
 static void stage5_dialog_post_boss(void) {
 	PlayerMode *pm = global.plr.mode;
