@@ -2,15 +2,13 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
-*/
-
-#include "taisei.h"
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
+ */
 
 #include "spells.h"
-#include "stagetext.h"
-#include "common_tasks.h"
+
+#include "../draw.h"
 
 #define LASER_LENGTH 60
 
@@ -21,7 +19,7 @@ void elly_spellbg_toe(Boss *b, int t) {
 		.pos = { VIEWPORT_W / 2.0, VIEWPORT_H / 2.0 },
 		.scale.both = 0.75 + 0.0005 * t,
 		.rotation.angle = t * 0.1 * DEG2RAD,
-		.sprite = "stage6/spellbg_toe",
+		.sprite_ptr = res_sprite("stage6/spellbg_toe"),
 		.color = RGB(0.6, 0.6, 0.6),
 	});
 
@@ -58,7 +56,7 @@ void elly_spellbg_toe(Boss *b, int t) {
 
 		SpriteParams sp = {
 			.color = &color,
-			.sprite = texname,
+			.sprite_ptr = res_sprite(texname),
 			.pos = {
 				VIEWPORT_W / 2.0 + positions[i][0] + cos(wobble + i) * wobble,
 				VIEWPORT_H / 2.0 - 150 + positions[i][1] + sin(i + wobble) * wobble,
