@@ -10,10 +10,10 @@
 #include "taisei.h"
 
 #include <png.h>
-#include <SDL.h>
+#include <SDL3/SDL.h>
 
-void pngutil_init_rwops_read(png_structp png, SDL_RWops *rwops);
-void pngutil_init_rwops_write(png_structp png, SDL_RWops *rwops);
+void pngutil_init_rwops_read(png_structp png, SDL_IOStream *rwops);
+void pngutil_init_rwops_write(png_structp png, SDL_IOStream *rwops);
 void pngutil_setup_error_handlers(png_structp png);
 png_structp pngutil_create_read_struct(void);
 png_structp pngutil_create_write_struct(void);
