@@ -2,8 +2,8 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
 #pragma once
@@ -11,43 +11,47 @@
 
 #include "geometry.h"
 #include "list.h"
-#include "memory/allocator.h"
+#include "memory/mempool.h"
 
 typedef struct RectPack RectPack;
 typedef struct RectPackSection RectPackSection;
+typedef MEMPOOL(RectPackSection) RectPackSectionPool;
+typedef struct RectPackSectionSource RectPackSectionSource;
 
 struct RectPackSection {
 	LIST_INTERFACE(RectPackSection);
-	Rect rect;
 	RectPackSection *parent;
 	RectPackSection *sibling;
 	RectPackSection *children[2];
+	Rect rect;
 };
 
 struct RectPack {
 	RectPackSection root;
 	RectPackSection *unused_sections;
-	RectPackSection *sections_freelist;
-	Allocator *allocator;
 };
 
-void rectpack_init(RectPack *rp, Allocator *alloc, double width, double height)
+struct RectPackSectionSource {
+	RectPackSectionPool *pool;
+	MemArena *arena;
+};
+
+void rectpack_init(RectPack *rp, double width, double height)
 	attr_nonnull_all;
 
-void rectpack_reset(RectPack *rp)
-	attr_nonnull(1);
-
-void rectpack_deinit(RectPack *rp)
-	attr_nonnull(1);
-
-RectPackSection *rectpack_add(RectPack *rp, double width, double height)
-	attr_nonnull(1);
+RectPackSection *rectpack_add(
+	RectPack *rp,
+	RectPackSectionSource secsrc,
+	double width,
+	double height,
+	bool allow_rotation
+) attr_nonnull(1);
 
 Rect rectpack_section_rect(RectPackSection *s)
 	attr_nonnull(1);
 
-void rectpack_reclaim(RectPack *rp, RectPackSection *s)
-	attr_nonnull(1, 2);
+void rectpack_reclaim(RectPack *rp, RectPackSectionSource secsrc, RectPackSection *s)
+	attr_nonnull(1, 3);
 
 bool rectpack_is_empty(RectPack *rp)
 	attr_nonnull(1);
