@@ -2,9 +2,9 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
-*/
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
+ */
 
 #pragma once
 #include "taisei.h"
@@ -29,7 +29,7 @@ void strbuf_clear(StringBuffer *strbuf)
 void strbuf_free(StringBuffer *strbuf)
 	attr_nonnull(1);
 
-int strbuf_ncat(StringBuffer *strbuf, size_t datasize, const char data[datasize])
+int strbuf_ncat(StringBuffer *strbuf, size_t datasize, const char data[])
 	attr_nonnull(1, 3);
 
 INLINE int strbuf_cat(StringBuffer *strbuf, const char *str) {
