@@ -12,13 +12,11 @@
 #include "events.h"
 #include "global.h"
 #include "renderer/api.h"
-#include "rwops/rwops_autobuf.h"
 #include "stagedraw.h"
 #include "taskmanager.h"
 #include "util/env.h"
 #include "util/fbmgr.h"
 #include "util/graphics.h"
-#include "util/io.h"
 #include "version.h"
 #include "video_postprocess.h"
 
@@ -38,7 +36,6 @@ static struct {
 	VideoMode current;
 	VideoBackend backend;
 	double scaling_factor;
-	uint num_resize_events;
 
 	struct {
 		char *name_prefix;
@@ -47,6 +44,9 @@ static struct {
 		int compression;
 		FramedumpSource source;
 	} framedump;
+
+	SDL_DisplayID *displays;
+	int num_displays;
 } video;
 
 #define FRAMEDUMP_FILENAME_SUFFIX ".png"
@@ -186,6 +186,12 @@ static VideoCapabilityState video_query_capability_switch(VideoCapability cap) {
 static VideoCapabilityState video_query_capability_webcanvas(VideoCapability cap) {
 	switch(cap) {
 		case VIDEO_CAP_EXTERNAL_RESIZE:
+			return VIDEO_ALWAYS_ENABLED;
+
+		case VIDEO_CAP_FULLSCREEN:
+			return VIDEO_NEVER_AVAILABLE;
+
+		case VIDEO_CAP_CHANGE_RESOLUTION:
 			return VIDEO_NEVER_AVAILABLE;
 
 		case VIDEO_CAP_VSYNC_ADAPTIVE:
@@ -268,7 +274,9 @@ static int video_compare_modes(const void *a, const void *b) {
 }
 
 static IntExtent video_get_screen_framebuffer_size(void) {
-	return r_framebuffer_get_size(NULL);
+	IntExtent s;
+	SDL_GetWindowSizeInPixels(video.window, &s.w, &s.h);
+	return s;
 }
 
 static FloatExtent video_get_viewport_size_for_framebuffer(IntExtent framebuffer_size) {
@@ -285,6 +293,34 @@ static FloatExtent video_get_viewport_size_for_framebuffer(IntExtent framebuffer
 	return (FloatExtent) { w, h };
 }
 
+static void video_get_viewport_for_framebuffer(FloatRect *vp, IntExtent framebuffer_size) {
+	vp->extent = video_get_viewport_size_for_framebuffer(framebuffer_size);
+	vp->x = (int)((framebuffer_size.w - vp->w) * 0.5);
+	vp->y = (int)((framebuffer_size.h - vp->h) * 0.5);
+}
+
+void video_get_viewport_size(float *width, float *height) {
+	IntExtent fb = video_get_screen_framebuffer_size();
+	FloatExtent vp = video_get_viewport_size_for_framebuffer(fb);
+
+	*width = vp.w;
+	*height = vp.h;
+}
+
+void video_get_viewport(FloatRect *vp) {
+	IntExtent fb = video_get_screen_framebuffer_size();
+	video_get_viewport_for_framebuffer(vp, fb);
+	log_debug("current w/h: %dx%d", video.current.width, video.current.height);
+	log_debug("viewport x/y: %fx%f", vp->x, vp->y);
+	log_debug("viewport w/h: %fx%f", vp->w, vp->h);
+}
+
+static void video_set_viewport(void) {
+	FloatRect vp;
+	video_get_viewport(&vp);
+	r_framebuffer_viewport_rect(NULL, vp);
+}
+
 static IntExtent round_viewport_size(FloatExtent vp) {
 	return (IntExtent) { round(vp.w), round(vp.h) };
 }
@@ -305,43 +341,49 @@ static void video_update_mode_lists(void) {
 
 	// Register all resolutions that are available in fullscreen and their corresponding windowed modes.
 	for(int s = 0; s < video_num_displays(); ++s) {
+		SDL_DisplayID display = video.displays[s];
 		log_info("Found display #%i: %s", s, video_display_name(s));
 
-		SDL_DisplayMode desktop_mode;
+		const SDL_DisplayMode *desktop_mode;
 		IntExtent screenspace_max_size = { 0 };
 
-		if(SDL_GetDesktopDisplayMode(s, &desktop_mode)) {
+		if(!(desktop_mode = SDL_GetDesktopDisplayMode(display))) {
 			log_sdl_error(LOG_WARN, "SDL_GetDesktopDisplayMode");
 		} else {
-			log_debug("Desktop mode: %ix%i@%iHz", desktop_mode.w, desktop_mode.h, desktop_mode.refresh_rate);
-			screenspace_max_size.w = desktop_mode.w;
-			screenspace_max_size.h = desktop_mode.h;
-			screenspace_max_size = coords_ext_pixels_to_screen(screenspace_max_size);
-			log_debug("Scaled screen-space bounds: %ix%i", screenspace_max_size.w, screenspace_max_size.h);
+			log_debug("Desktop mode: %ix%i@%gHz, scale: %g",
+				desktop_mode->w, desktop_mode->h,
+				desktop_mode->refresh_rate, desktop_mode->pixel_density
+			);
+			screenspace_max_size.w = desktop_mode->w;
+			screenspace_max_size.h = desktop_mode->h;
 		}
 
-		for(int i = 0; i < SDL_GetNumDisplayModes(s); ++i) {
-			SDL_DisplayMode mode = { SDL_PIXELFORMAT_UNKNOWN, 0, 0, 0, 0 };
+		int mcount;
+		SDL_DisplayMode **modes = SDL_GetFullscreenDisplayModes(display, &mcount);
 
-			if(SDL_GetDisplayMode(s, i, &mode) != 0) {
-				log_sdl_error(LOG_WARN, "SDL_GetDisplayMode");
-			} else {
-				log_debug("Display mode #%i: %ix%i@%iHz", i, mode.w, mode.h, mode.refresh_rate);
+		for(int i = 0; i < mcount; ++i) {
+			const SDL_DisplayMode *mode = modes[i];
 
-				video_add_mode_fullscreen((IntExtent) { mode.w, mode.h }, screenspace_min_size, screenspace_max_size);
-				fullscreen_available = true;
+			log_debug("Display mode #%i: %ix%i@%gHz; scale = %g", i,
+				mode->w, mode->h, mode->refresh_rate, mode->pixel_density);
 
-				if(has_windowed_modes) {
-					FloatExtent vp = video_get_viewport_size_for_framebuffer((IntExtent) { mode.w, mode.h });
-					video_add_mode_windowed(round_viewport_size(vp), screenspace_min_size, screenspace_max_size);
+			video_add_mode_fullscreen((IntExtent) { mode->w, mode->h }, screenspace_min_size, screenspace_max_size);
+			fullscreen_available = true;
 
-					// the ratio is always constant, so we need to check only 1 dimension
-					if(vp.w > largest_fullscreen_viewport.w) {
-						largest_fullscreen_viewport = vp;
-					}
+			if(has_windowed_modes) {
+				FloatExtent vp = video_get_viewport_size_for_framebuffer(
+					(IntExtent) { mode->w, mode->h });
+				video_add_mode_windowed(
+					round_viewport_size(vp), screenspace_min_size, screenspace_max_size);
+
+				// the ratio is always constant, so we need to check only 1 dimension
+				if(vp.w > largest_fullscreen_viewport.w) {
+					largest_fullscreen_viewport = vp;
 				}
 			}
 		}
+
+		SDL_free(modes);
 	}
 
 	if(has_windowed_modes) {
@@ -387,8 +429,7 @@ static void video_update_mode_lists(void) {
 }
 
 static void video_update_scaling_factor(void) {
-	// NOTE: must query the main framebuffer explicitly here; postprocess buffers may have outdated information.
-	IntExtent main_fb = r_framebuffer_get_size(NULL);
+	IntExtent main_fb = video_get_screen_framebuffer_size();
 	assert(main_fb.w > 0);
 	double scaling_factor = (double)main_fb.w / video.current.width;
 
@@ -402,35 +443,6 @@ static void video_update_scaling_factor(void) {
 	}
 }
 
-void video_get_viewport_size(float *width, float *height) {
-	IntExtent fb = video_get_screen_framebuffer_size();
-	FloatExtent vp = video_get_viewport_size_for_framebuffer(fb);
-
-	*width = vp.w;
-	*height = vp.h;
-}
-
-void video_get_viewport(FloatRect *vp) {
-	IntExtent fb = video_get_screen_framebuffer_size();
-
-	// vp->extent aliases vp->w and vp->h; see util/geometry.h
-	vp->extent = video_get_viewport_size_for_framebuffer(fb);
-
-	vp->x = (int)((fb.w - vp->w) * 0.5);
-	vp->y = (int)((fb.h - vp->h) * 0.5);
-
-	// This function can also be changed to return a FloatRect instead
-	log_debug("current w/h: %dx%d", video.current.width, video.current.height);
-	log_debug("viewport x/y: %fx%f", vp->x, vp->y);
-	log_debug("viewport w/h: %fx%f", vp->w, vp->h);
-}
-
-static void video_set_viewport(void) {
-	FloatRect vp;
-	video_get_viewport(&vp);
-	r_framebuffer_viewport_rect(NULL, vp);
-}
-
 static void video_update_vsync(void) {
 	if(global.frameskip || config_get_int(CONFIG_VSYNC) == 0) {
 		r_vsync(VSYNC_NONE);
@@ -443,16 +455,23 @@ static void video_update_vsync(void) {
 }
 
 static void video_update_mode_settings(void) {
-	SDL_ShowCursor(!video_is_fullscreen());
-	video_update_vsync();
+	if(video_is_fullscreen()) {
+		SDL_HideCursor();
+	} else {
+		SDL_ShowCursor();
+	}
+
 	SDL_GetWindowSize(video.window, &video.current.width, &video.current.height);
-	video_set_viewport();
+
+	video_update_vsync();
 	video_update_scaling_factor();
+	video_set_viewport();
+
 	events_emit(TE_VIDEO_MODE_CHANGED, 0, NULL, NULL);
 }
 
 static const char *modeflagsstr(uint32_t flags) {
-	if(flags & SDL_WINDOW_FULLSCREEN_DESKTOP) {
+	if(flags & SDL_WINDOW_FULLSCREEN) {
 		return "fullscreen";
 	} else if(flags & SDL_WINDOW_RESIZABLE) {
 		return "windowed, resizable";
@@ -466,7 +485,6 @@ static void video_new_window_internal(uint display, uint w, uint h, uint32_t fla
 		r_unclaim_window(video.window);
 		SDL_DestroyWindow(video.window);
 		video.window = NULL;
-		video.num_resize_events = 0;
 	}
 
 	char title[sizeof(WINDOW_TITLE) + strlen(TAISEI_VERSION) + 2];
@@ -495,15 +513,33 @@ static void video_new_window_internal(uint display, uint w, uint h, uint32_t fla
 		return;
 	}
 
-	video_new_window_internal(display, RESX, RESY, flags & ~SDL_WINDOW_FULLSCREEN_DESKTOP, true);
+	video_new_window_internal(display, RESX, RESY, flags & ~SDL_WINDOW_FULLSCREEN, true);
+}
+
+static bool restrict_to_capability(bool enabled, VideoCapability cap) {
+	VideoCapabilityState capval = video_query_capability(cap);
+
+	switch(capval) {
+		case VIDEO_ALWAYS_ENABLED:
+			return true;
+
+		case VIDEO_NEVER_AVAILABLE:
+			return false;
+
+		default:
+			return enabled;
+	}
 }
 
 static void video_new_window(uint display, uint w, uint h, bool fs, bool resizable) {
-	uint32_t flags = SDL_WINDOW_ALLOW_HIGHDPI;
+	uint32_t flags = SDL_WINDOW_HIGH_PIXEL_DENSITY;
 
-	if(fs || video_query_capability(VIDEO_CAP_FULLSCREEN) == VIDEO_ALWAYS_ENABLED) {
-		flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
-	} else if(resizable && video.backend != VIDEO_BACKEND_EMSCRIPTEN) {
+	fs = restrict_to_capability(fs, VIDEO_CAP_FULLSCREEN);
+	resizable = restrict_to_capability(resizable, VIDEO_CAP_EXTERNAL_RESIZE);
+
+	if(fs) {
+		flags |= SDL_WINDOW_FULLSCREEN;
+	} else if(resizable) {
 		flags |= SDL_WINDOW_RESIZABLE;
 	}
 
@@ -523,10 +559,10 @@ static void video_new_window(uint display, uint w, uint h, bool fs, bool resizab
 }
 
 static void video_set_fullscreen_internal(bool fullscreen) {
-	uint32_t flags = fullscreen ? SDL_WINDOW_FULLSCREEN_DESKTOP : 0;
+	uint32_t flags = fullscreen ? SDL_WINDOW_FULLSCREEN : 0;
 	events_pause_keyrepeat();
 
-	if(SDL_SetWindowFullscreen(video.window, flags) < 0) {
+	if(!SDL_SetWindowFullscreen(video.window, flags)) {
 		log_error("Failed to switch to %s mode: %s", modeflagsstr(flags), SDL_GetError());
 	}
 
@@ -538,7 +574,7 @@ INLINE bool should_recreate_on_size_change(void) {
 		/* Resize failures are impossible to detect under some WMs */
 		video.backend == VIDEO_BACKEND_X11 ||
 		/* Needed to work around various SDL bugs and/or HTML/DOM quirks */
-		video.backend == VIDEO_BACKEND_EMSCRIPTEN ||
+		// video.backend == VIDEO_BACKEND_EMSCRIPTEN ||
 	0);
 
 	return env_get("TAISEI_VIDEO_RECREATE_ON_RESIZE", defaultval);
@@ -553,20 +589,6 @@ INLINE bool should_recreate_on_fullscreen_change(void) {
 	return env_get("TAISEI_VIDEO_RECREATE_ON_FULLSCREEN", defaultval);
 }
 
-static bool restrict_to_capability(bool enabled, VideoCapability cap) {
-	VideoCapabilityState capval = video_query_capability(cap);
-
-	switch(capval) {
-		case VIDEO_ALWAYS_ENABLED:
-			return true;
-
-		case VIDEO_NEVER_AVAILABLE:
-			return false;
-
-		default:
-			return enabled;
-	}
-}
 
 void video_set_mode(uint display, uint w, uint h, bool fs, bool resizable) {
 	fs = restrict_to_capability(fs, VIDEO_CAP_FULLSCREEN);
@@ -585,6 +607,14 @@ void video_set_mode(uint display, uint w, uint h, bool fs, bool resizable) {
 		return;
 	}
 
+	if(
+		!restrict_to_capability(true, VIDEO_CAP_CHANGE_RESOLUTION) &&
+		video.current.width > 0 && video.current.height > 0
+	) {
+		w = video.current.width;
+		h = video.current.height;
+	}
+
 	bool display_changed = display != video_current_display();
 	bool size_changed = w != video.current.width || h != video.current.height;
 	bool fullscreen_changed = video_is_fullscreen() != fs;
@@ -599,7 +629,7 @@ void video_set_mode(uint display, uint w, uint h, bool fs, bool resizable) {
 		return;
 	}
 
-	if(size_changed) {
+	if(size_changed && !fs) {
 		if(!fullscreen_changed && should_recreate_on_size_change()) {
 			video_new_window(display, w, h, fs, resizable);
 			return;
@@ -626,8 +656,11 @@ SDL_Window *video_get_window(void) {
 }
 
 void video_set_fullscreen(bool fullscreen) {
-	video_set_mode(
-		SDL_GetWindowDisplayIndex(video.window),
+	if(video_query_capability(VIDEO_CAP_FULLSCREEN) != VIDEO_AVAILABLE) {
+		return;
+	}
+
+	video_set_mode(video_current_display(),
 		video.intended.width,
 		video.intended.height,
 		fullscreen,
@@ -665,7 +698,7 @@ static void *video_screenshot_task(void *arg) {
 		char buf[video.framedump.name_prefix_len + FRAMEDUMP_FILENAME_EXTRA_BUFSIZE];
 		snprintf(buf, sizeof(buf), "%s" FRAMEDUMP_FILENAME_FORMAT, video.framedump.name_prefix, tdata->frame_num);
 
-		SDL_RWops *stream = SDL_RWFromFile(buf, "wb");
+		SDL_IOStream *stream = SDL_IOFromFile(buf, "wb");
 
 		if(UNLIKELY(!stream)) {
 			log_sdl_error(LOG_ERROR, "SDL_RWFromFile");
@@ -674,7 +707,7 @@ static void *video_screenshot_task(void *arg) {
 
 		opts.zlib_compression_level = video.framedump.compression;
 		bool ok = pixmap_save_stream(stream, &tdata->image, &opts.base);
-		SDL_RWclose(stream);
+		SDL_CloseIO(stream);
 
 		if(LIKELY(ok)) {
 			log_debug("Frame dump: %s", buf);
@@ -696,6 +729,7 @@ static void video_screenshot_free_task_data(void *arg) {
 static void video_take_screenshot_callback(const Pixmap *px, void *userdata) {
 	if(!px) {
 		log_error("Failed to capture image");
+		video_screenshot_free_task_data(userdata);
 		return;
 	}
 
@@ -711,6 +745,20 @@ static void video_take_screenshot_callback(const Pixmap *px, void *userdata) {
 	}));
 }
 
+static void video_read_framebuffer(
+	Framebuffer *fb,
+	FramebufferAttachment attachment,
+	void *data,
+	FramebufferReadAsyncCallback callback
+) {
+	if(!fb && !(r_features() & r_feature_bit(RFEAT_DEFAULT_FRAMEBUFFER_READBACK))) {
+		log_error("Renderer doesn't support reading from the default framebuffer");
+		callback(NULL, data);
+	} else {
+		r_framebuffer_read_viewport_async(fb, attachment, data, callback);
+	}
+}
+
 void video_take_screenshot(bool viewport_only) {
 	auto tdata = ALLOC(ScreenshotTaskData);
 
@@ -728,8 +776,7 @@ void video_take_screenshot(bool viewport_only) {
 		attachment = FRAMEBUFFER_ATTACH_COLOR0;
 	}
 
-	r_framebuffer_read_viewport_async(
-		fb, attachment, tdata, video_take_screenshot_callback);
+	video_read_framebuffer(fb, attachment, tdata, video_take_screenshot_callback);
 }
 
 static void video_take_framedump(void) {
@@ -748,8 +795,7 @@ static void video_take_framedump(void) {
 	auto tdata = ALLOC(ScreenshotTaskData);
 	tdata->frame_num = video.framedump.frame_count++;
 
-	r_framebuffer_read_viewport_async(
-		fb, attachment, tdata, video_take_screenshot_callback);
+	video_read_framebuffer(fb, attachment, tdata, video_take_screenshot_callback);
 }
 
 bool video_is_resizable(void) {
@@ -757,129 +803,26 @@ bool video_is_resizable(void) {
 }
 
 bool video_is_fullscreen(void) {
-	return SDL_GetWindowFlags(video.window) & SDL_WINDOW_FULLSCREEN_DESKTOP;
+	return SDL_GetWindowFlags(video.window) & SDL_WINDOW_FULLSCREEN;
 }
 
 static void video_init_sdl(void) {
 	// XXX: workaround for an SDL bug: https://bugzilla.libsdl.org/show_bug.cgi?id=4127
+	// TODO: test if still needed for SDL3 (but in any case it won't hurt)
 	SDL_SetHintWithPriority(SDL_HINT_FRAMEBUFFER_ACCELERATION, "0", SDL_HINT_OVERRIDE);
 
-	uint num_drivers = SDL_GetNumVideoDrivers();
-	const char *video_drivers[num_drivers];
-
-	void *buf;
-	SDL_RWops *out = SDL_RWAutoBuffer(&buf, 256);
-
-	SDL_RWprintf(out, "Available video drivers:");
-
-	for(uint i = 0; i < num_drivers; ++i) {
-		video_drivers[i] = SDL_GetVideoDriver(i);
-		SDL_RWprintf(out, " %s", video_drivers[i]);
-	}
-
-	SDL_WriteU8(out, 0);
-	log_info("%s", (char*)buf);
-	SDL_RWclose(out);
-
-	// https://bugzilla.libsdl.org/show_bug.cgi?id=3948
-	// A suboptimal X11 server may be available on top of those systems,
-	// so we push X11 down in the priority list.
-	const char *prefer_drivers = env_get_string_nonempty("TAISEI_PREFER_SDL_VIDEODRIVERS", "wayland,cocoa,windows,x11");
-	const char *force_driver = env_get_string_nonempty("TAISEI_VIDEO_DRIVER", NULL);
-
-	if(force_driver) {
-		log_warn("TAISEI_VIDEO_DRIVER is deprecated and will be removed, use TAISEI_PREFER_SDL_VIDEODRIVERS or SDL_VIDEODRIVER instead");
-	} else {
-		force_driver = env_get_string_nonempty("SDL_VIDEODRIVER", NULL);
-	}
-
-	if(prefer_drivers && *prefer_drivers && !force_driver) {
-		char buf[strlen(prefer_drivers) + 1];
-		char *driver, *bufptr = buf;
-		int drivernum = 0;
-		strcpy(buf, prefer_drivers);
-
-		while((driver = strtok_r(NULL, " :;,", &bufptr))) {
-			bool skip = true;
-
-			for(uint i = 0; i < num_drivers; ++i) {
-				if(!strcmp(video_drivers[i], driver)) {
-					skip = false;
-					break;
-				}
-			}
-
-			++drivernum;
-
-			if(skip) {
-				continue;
-			}
-
-			log_info("Trying preferred driver #%i: %s", drivernum, driver);
-
-			if(SDL_VideoInit(driver)) {
-				log_info("Driver '%s' failed: %s", driver, SDL_GetError());
-			} else {
-				return;
-			}
-		}
-
-		if(drivernum) {
-			log_info("All preferred drivers failed, falling back to SDL default");
-		}
-	}
-
-	if(SDL_VideoInit(force_driver)) {
-		log_fatal("SDL_VideoInit() failed: %s", SDL_GetError());
+	if(!SDL_InitSubSystem(SDL_INIT_VIDEO)) {
+		log_sdl_error(LOG_FATAL, "SDL_InitSubSystem");
 	}
 }
 
-static void video_handle_resize(int w, int h) {
-	int minw, minh;
-	SDL_GetWindowMinimumSize(video.window, &minw, &minh);
+static void video_update_displays(void) {
+	SDL_free(video.displays);
 
-	if((w < minw || h < minh) && video.num_resize_events > 3) {
-		log_warn("Bad resize: %ix%i is too small!", w, h);
-		// FIXME: the video_new_window is actually a workaround for Wayland.
-		// I'm not sure if it's necessary for anything else.
-		video_new_window(video_current_display(), video.intended.width, video.intended.height, false, video_is_resizable());
-		return;
+	if(!(video.displays = SDL_GetDisplays(&video.num_displays))) {
+		log_sdl_error(LOG_ERROR, "SDL_GetDisplays");
+		video.num_displays = 0;
 	}
-
-	log_debug("%ix%i --> %ix%i", video.current.width, video.current.height, w, h);
-	video.current.width = w;
-	video.current.height = h;
-	video_update_mode_settings();
-	++video.num_resize_events;
-}
-
-static bool video_handle_window_event(SDL_Event *event, void *arg) {
-	switch(event->window.event) {
-		case SDL_WINDOWEVENT_SIZE_CHANGED:
-			// This event is generated for any resizes, including calls to SDL_SetWindowSize.
-			// It's followed by SDL_WINDOWEVENT_RESIZED for external resizes (from the WM or the
-			// user). We only need to handle external resizes.
-			log_debug("SDL_WINDOWEVENT_SIZE_CHANGED: %ix%i", event->window.data1, event->window.data2);
-
-			// Catch resizes by the SDL portlib itself, when the console is docked/undocked
-			// https://github.com/devkitPro/SDL/issues/31
-			if(video_get_backend() == VIDEO_BACKEND_SWITCH) {
-				video_handle_resize(event->window.data1, event->window.data2);
-			}
-			break;
-
-		case SDL_WINDOWEVENT_RESIZED:
-			video_handle_resize(event->window.data1, event->window.data2);
-			break;
-
-		case SDL_WINDOWEVENT_FOCUS_LOST:
-			if(config_get_int(CONFIG_FOCUS_LOSS_PAUSE)) {
-				events_emit(TE_GAME_PAUSE, 0, NULL, NULL);
-			}
-			break;
-	}
-
-	return true;
 }
 
 static bool video_handle_config_event(SDL_Event *evt, void *arg) {
@@ -906,44 +849,64 @@ static bool video_handle_config_event(SDL_Event *evt, void *arg) {
 	return false;
 }
 
-uint video_num_displays(void) {
-	int displays = SDL_GetNumVideoDisplays();
+static bool video_handle_event(SDL_Event *event, void *arg) {
+	switch(event->type) {
+		case SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED:
+			log_debug("SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED: %ix%i", event->window.data1, event->window.data2);
+			video_update_mode_settings();
+			break;
 
-	if(displays < 1) {
-		if(displays == 0) {
-			log_warn("SDL_GetNumVideoDisplays() returned 0, this shouldn't happen");
-		} else {
-			log_sdl_error(LOG_WARN, "SDL_GetNumVideoDisplays");
-		}
+		case SDL_EVENT_WINDOW_FOCUS_LOST:
+			if(config_get_int(CONFIG_FOCUS_LOSS_PAUSE)) {
+				events_emit(TE_GAME_PAUSE, 0, NULL, NULL);
+			}
+			break;
 
-		displays = 1;
+		case SDL_EVENT_DISPLAY_ADDED:
+		case SDL_EVENT_DISPLAY_REMOVED:
+			video_update_displays();
+			break;
+
+		default:
+			if(event->type == MAKE_TAISEI_EVENT(TE_CONFIG_UPDATED)) {
+				return video_handle_config_event(event, arg);
+			}
 	}
 
-	return displays;
+	return false;
+}
+
+uint video_num_displays(void) {
+	return video.num_displays;
 }
 
 const char *video_display_name(uint id) {
-	assert(id < video_num_displays());
-
-	const char *name = SDL_GetDisplayName(id);
+	const char *name = SDL_GetDisplayName(video.displays[id]);
 
 	if(name == NULL) {
 		log_sdl_error(LOG_WARN, "SDL_GetDisplayName");
-		name = "Unknown";
+		return "Unknown";
 	}
 
 	return name;
 }
 
 uint video_current_display(void) {
-	int display = SDL_GetWindowDisplayIndex(video.window);
+	SDL_DisplayID display = SDL_GetDisplayForWindow(video.window);
 
-	if(display < 0) {
-		log_sdl_error(LOG_WARN, "SDL_GetWindowDisplayIndex");
-		display = 1;
+	if(!display) {
+		log_sdl_error(LOG_WARN, "SDL_GetDisplayForWindow");
+		return 0;  // FIXME return error?
 	}
 
-	return display;
+	for(int i = 0; i < video.num_displays; ++i) {
+		if(video.displays[i] == display) {
+			return i;
+		}
+	}
+
+	assert(0 && "Display index for window not found");
+	return 0;
 }
 
 static void video_init_framedump(void) {
@@ -1029,6 +992,8 @@ void video_init(const VideoInitParams *params) {
 		fullscreen = config_get_int(CONFIG_FULLSCREEN);
 	}
 
+	video_update_displays();
+
 	video_set_mode(
 		config_get_int(CONFIG_VID_DISPLAY),
 		w, h, fullscreen,
@@ -1038,15 +1003,8 @@ void video_init(const VideoInitParams *params) {
 	video_update_scaling_factor();
 
 	events_register_handler(&(EventHandler) {
-		.proc = video_handle_window_event,
+		.proc = video_handle_event,
 		.priority = EPRIO_SYSTEM,
-		.event_type = SDL_WINDOWEVENT,
-	});
-
-	events_register_handler(&(EventHandler) {
-		.proc = video_handle_config_event,
-		.priority = EPRIO_SYSTEM,
-		.event_type = MAKE_TAISEI_EVENT(TE_CONFIG_UPDATED),
 	});
 
 	video_init_framedump();
@@ -1062,14 +1020,14 @@ void video_post_init(void) {
 void video_shutdown(void) {
 	video_postprocess_shutdown(video.postprocess);
 	fbmgr_shutdown();
-	events_unregister_handler(video_handle_window_event);
-	events_unregister_handler(video_handle_config_event);
+	events_unregister_handler(video_handle_event);
 	r_unclaim_window(video.window);
 	SDL_DestroyWindow(video.window);
 	r_shutdown();
+	SDL_free(video.displays);
 	dynarray_free_data(&video.win_modes);
 	dynarray_free_data(&video.fs_modes);
-	SDL_VideoQuit();
+	SDL_QuitSubSystem(SDL_INIT_VIDEO);
 	mem_free(video.framedump.name_prefix);
 }
 
